@@ -1,5 +1,5 @@
 #
-# $Id: BDB.pm,v 1.4 2002/02/28 08:26:54 heikki Exp $
+# $Id: BDB.pm,v 1.6.2.1 2003/03/25 18:46:10 jason Exp $
 #
 # BioPerl module for Bio::DB::Flat::BDB
 #
@@ -15,7 +15,7 @@ Bio::DB::Flat::BDB - Interface for BioHackathon standard BDB-indexed flat file
 
 =head1 SYNOPSIS
 
-You should not be using this module directly
+You should not be using this module directly.  See Bio::DB::Flat.
 
 =head1 DESCRIPTION
 
@@ -45,13 +45,16 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via
 email or the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Lincoln Stein
 
 Email - lstein@cshl.org
 
+=head1 SEE ALSO
+
+L<Bio::DB::Flat>,
+
 =head1 APPENDIX
 
 The rest of the documentation details each of the object methods. Internal
@@ -130,7 +133,19 @@ sub get_Seq_by_id {
 # fetch array of Bio::Seq objects
 sub get_Seq_by_acc {
   my $self = shift;
-  return $self->get_Seq_by_id(shift) if @_ == 1;
+  unshift @_,'ACC' if @_==1;
+  my ($ns,$key) = @_;
+  my @primary_ids = $self->expand_ids($ns => $key);
+  $self->throw("more than one sequences correspond to this accession")
+      if @primary_ids > 1 && ! wantarray;
+  my @rc = map {$self->get_Seq_by_id($_)} @primary_ids;
+ return wantarray ? @rc : $rc[0];
+}
+
+# fetch array of Bio::Seq objects
+sub get_Seq_by_version {
+  my $self = shift;
+  unshift @_,'VERSION' if @_==1;
   my ($ns,$key) = @_;
   my @primary_ids = $self->expand_ids($ns => $key);
   $self->throw("more than one sequences correspond to this accession")
@@ -195,7 +210,9 @@ sub build_index {
   my @files = @_;
   my $count = 0;
   for my $file (@files) {
-    $count++ if $self->_index_file($file);
+    $file = File::Spec->rel2abs($file)
+      unless File::Spec->file_name_is_absolute($file);
+    $count += $self->_index_file($file);
   }
   $self->write_config;
   $count;
@@ -210,14 +227,16 @@ sub _index_file {
 
   my $fh     = $self->_fhcache($file) or $self->throw("could not open $file for indexing: $!");
   my $offset = 0;
+  my $count  = 0;
   while (!eof($fh)) {
-    my ($ids,$adjustment)  = $self->parse_one_record($fh);
+    my ($ids,$adjustment)  = $self->parse_one_record($fh) or next;
     $adjustment ||= 0;  # prevent uninit variable warning
     my $pos = tell($fh) + $adjustment;
     $self->_store_index($ids,$file,$offset,$pos-$offset);
     $offset = $pos;
+    $count++;
   }
-  1;
+  $count;
 }
 
 =head2 To Be Implemented in Subclasses
@@ -400,7 +419,7 @@ sub _open_bdb {
 
   my $primary_db = {};
   tie(%$primary_db,'DB_File',$self->_catfile($self->_primary_db_name),$flags,0666,$DB_BTREE)
-    or $self->throw("Could not open primary index file");
+    or $self->throw("Could not open primary index file: $! (did you remember to use -write_flag=>1?)");
   $self->{bdb_primary_db} = $primary_db;
 
   for my $secondary ($self->secondary_namespaces) {
