@@ -1,6 +1,41 @@
+=head1 NAME
+
+Bio::DB::GFF::Adaptor::memory_iterator - iterator for Bio::DB::GFF::Adaptor::memory
+
+=head1 SYNOPSIS
+
+For internal use only
+
+=head1 DESCRIPTION
+
+This is an internal module that is used by the Bio::DB::GFF in-memory
+adaptor to return an iterator across a sequence feature query.  The
+object has a single method, next_feature(), that returns the next
+feature from the query.  The method next_seq() is an alias for
+next_feature().
+
+=head1 BUGS
+
+None known yet.
+
+=head1 SEE ALSO
+
+L<Bio::DB::GFF>,
+
+=head1 AUTHOR
+
+Lincoln Stein E<lt>lstein@cshl.orgE<gt>.
+
+Copyright (c) 2001 Cold Spring Harbor Laboratory.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+=cut
+
 package Bio::DB::GFF::Adaptor::memory_iterator;
 use strict;
-# $Id: memory_iterator.pm,v 1.1.2.3 2002/06/19 21:57:27 lstein Exp $
+# $Id: memory_iterator.pm,v 1.4 2002/09/27 01:36:48 lstein Exp $
 # this module needs to be cleaned up and documented
 
 #use constant STH         => 0;
