@@ -5,6 +5,9 @@ use Bio::Graphics::Glyph;
 use vars '@ISA';
 @ISA = 'Bio::Graphics::Glyph';
 
+my %complement = (g=>'c',a=>'t',t=>'a',c=>'g',
+		  G=>'C',A=>'T',T=>'A',C=>'G');
+
 # new options are 'label'       -- short label to print over glyph
 #                 'description'  -- long label to print under glyph
 # label and description can be flags or coderefs.
@@ -43,15 +46,17 @@ sub labelheight {
 }
 sub label {
   my $self = shift;
+  return if $self->{overbumped};  # set by the bumper when we have hit bump limit
   return unless $self->{level} == 0;
   return exists $self->{label} ? $self->{label}
-                               : $self->{label} = $self->_label;
+                               : ($self->{label} = $self->_label);
 }
 sub description {
   my $self = shift;
+  return if $self->{overbumped}; # set by the bumper when we have hit bump limit
   return unless $self->{level} == 0;
   return exists $self->{description} ? $self->{description}
-                                     : $self->{description} = $self->_description;
+                                     : ($self->{description} = $self->_description);
 }
 sub _label {
   my $self = shift;
@@ -62,11 +67,14 @@ sub _label {
   return $label unless $label eq '1';
   return "1"    if $label eq '1 '; # 1 with a space
 
+
   # figure it out ourselves
   my $f = $self->feature;
-  my $info = eval {$f->info};
-  return $info if $info;
-  return eval {$f->seqname} || eval{$f->primary_tag};
+
+  return $f->info         if $f->can('info');
+  return $f->seq_id       if $f->can('seq_id');
+  return $f->display_name if $f->can('display_name');   # deprecated API
+  return eval{$f->primary_tag};
 }
 sub _description {
   my $self = shift;
@@ -84,9 +92,11 @@ sub _description {
 sub get_description {
   my $self = shift;
   my $feature = shift;
-  if (my @notes = eval { $feature->notes }) {
-    return join '; ',@notes;
-  }
+
+  # common places where we can get descriptions
+  return join '; ',$feature->notes if $feature->can('notes');
+  return $feature->desc            if $feature->can('desc');
+
   my $tag = $feature->source_tag;
   return undef if $tag eq '';
   $tag;
@@ -125,6 +135,16 @@ sub draw_description {
 	      $self->font2color);
 }
 
+sub dna_fits {
+  my $self = shift;
+
+  my $pixels_per_base = $self->scale;
+  my $font            = $self->font;
+  my $font_width      = $font->width;
+
+  return $pixels_per_base >= $font_width;
+}
+
 sub arrowhead {
   my $self = shift;
   my $gd   = shift;
@@ -168,6 +188,11 @@ sub arrow {
   $self->arrowhead($gd,$x2,$y,$height,-1) if $x2 < $x1;
 }
 
+sub reversec {
+  $_[1]=~tr/gatcGATC/ctagCTAG/;
+  return scalar reverse $_[1];
+}
+
 1;
 
 =head1 NAME
