@@ -1,7 +1,7 @@
-# $Id: FuzzyLocationI.pm,v 1.15 2002/01/08 01:25:31 jason Exp $
+# $Id: FuzzyLocationI.pm,v 1.17 2002/12/01 00:05:20 jason Exp $
 #
 # BioPerl module for Bio::Location::FuzzyLocationI
-# Cared for by Jason Stajich <jason@chg.mc.duke.edu>
+# Cared for by Jason Stajich <jason@bioperl.org>
 #
 # Copyright Jason Stajich
 #
@@ -46,11 +46,11 @@ the bugs and their resolution.  Bug reports can be submitted via email
 or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich
 
-Email jason@chg.mc.duke.edu
+Email jason@bioperl.org
 
 =head1 APPENDIX
 
