@@ -1,4 +1,4 @@
-# $Id: PrimarySeqI.pm,v 1.38.2.2 2002/03/15 12:31:41 heikki Exp $
+# $Id: PrimarySeqI.pm,v 1.50.2.1 2003/03/31 11:49:51 heikki Exp $
 #
 # BioPerl module for Bio::PrimarySeqI
 #
@@ -12,15 +12,21 @@
 
 =head1 NAME
 
-Bio::PrimarySeqI - Interface definition for a Bio::PrimarySeq
+Bio::PrimarySeqI [Developers] - Interface definition for a Bio::PrimarySeq
 
 =head1 SYNOPSIS
 
-    # get a Bio::PrimarySeqI compliant object somehow
+
+    # Bio::PrimarySeqI is the interface class for sequences.
+
+    # If you are a newcomer to bioperl, you should
+    # start with Bio::Seq documentation. This
+    # documentation is mainly for developers using
+    # Bioperl.
 
     # to test this is a seq object
 
-    $obj->isa("Bio::PrimarySeqI") || 
+    $obj->isa("Bio::PrimarySeqI") ||
       $obj->throw("$obj does not implement the Bio::PrimarySeqI interface");
 
     # accessors
@@ -28,7 +34,7 @@ Bio::PrimarySeqI - Interface definition for a Bio::PrimarySeq
     $string    = $obj->seq();
     $substring = $obj->subseq(12,50);
     $display   = $obj->display_id(); # for human display
-    $id        = $obj->primary_id(); # unique id for this object, 
+    $id        = $obj->primary_id(); # unique id for this object,
                                      # implementation defined
     $unique_key= $obj->accession_number();
                        # unique biological id
@@ -51,14 +57,22 @@ Bio::PrimarySeqI - Interface definition for a Bio::PrimarySeq
 =head1 DESCRIPTION
 
 This object defines an abstract interface to basic sequence
-information. PrimarySeq is an object just for the sequence and its
-name(s), nothing more. Seq is the larger object complete with
-features. There is a pure perl implementation of this in
-Bio::PrimarySeq. If you just want to use Bio::PrimarySeq objects, then
-please read that module first. This module defines the interface, and
-is of more interest to people who want to wrap their own Perl
-Objects/RDBs/FileSystems etc in way that they "are" bioperl sequence
-objects, even though it is not using Perl to store the sequence etc.
+information - for most users of the package the documentation (and
+methods) in this class are not useful - this is a developers only
+class which defines what methods have to be implmented by other Perl
+objects to comply to the Bio::PrimarySeqI interface. Go "perldoc
+Bio::Seq" or "man Bio::Seq" for more information on the main class for
+sequences.
+
+
+PrimarySeq is an object just for the sequence and its name(s), nothing
+more. Seq is the larger object complete with features. There is a pure
+perl implementation of this in Bio::PrimarySeq. If you just want to
+use Bio::PrimarySeq objects, then please read that module first. This
+module defines the interface, and is of more interest to people who
+want to wrap their own Perl Objects/RDBs/FileSystems etc in way that
+they "are" bioperl sequence objects, even though it is not using Perl
+to store the sequence etc.
 
 
 This interface defines what bioperl consideres necessary to "be" a
@@ -90,7 +104,7 @@ the bugs and their resolution.  Bug reports can be submitted via email
 or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Ewan Birney
 
@@ -113,7 +127,6 @@ use strict;
 use Bio::Root::RootI;
 use Bio::Tools::CodonTable;
 
-
 @ISA = qw(Bio::Root::RootI);
 
 =head1 Implementation Specific Functions
@@ -233,6 +246,8 @@ sub accession_number {
            For sequences with no accession number, this method should
            return a stringified memory location.
 
+           [Note this method name is likely to change in 1.3]
+
  Returns : A string
  Args    : None
  Status  : Virtual
@@ -404,7 +419,8 @@ sub revcom{
 			     '-display_id'  => $self->display_id,
 			     '-accession_number' => $self->accession_number,
 			     '-alphabet' => $self->alphabet,
-			     '-desc' => $self->desc()
+			     '-desc' => $self->desc(),
+                             '-verbose' => $self->verbose
 			     );
    return $out;
 
@@ -425,22 +441,22 @@ sub revcom{
 
 sub trunc{
    my ($self,$start,$end) = @_;
-   
+
    my $str;
    if( defined $start && ref($start) &&
        $start->isa('Bio::LocationI') ) {
        $str = $self->subseq($start); # start is a location actually
    } elsif( !$end ) {
-       $self->throw("trunc start,end");
+       $self->throw("trunc start,end -- there was no end for $start");
    } elsif( $end < $start ) {
-       my $msg = "start [$start] is greater than end [$end. \n".
+       my $msg = "start [$start] is greater than end [$end]. \n".
 	   "If you want to truncated and reverse complement, \n".
 	       "you must call trunc followed by revcom. Sorry.";
        $self->throw($msg);
-   } else { 
+   } else {
        $str = $self->subseq($start,$end);
    }
-   
+
    my $seqclass;
    if($self->can_call_new()) {
        $seqclass = ref($self);
@@ -453,7 +469,8 @@ sub trunc{
 			     '-display_id'  => $self->display_id,
 			     '-accession_number' => $self->accession_number,
 			     '-alphabet' => $self->alphabet,
-			     '-desc' => $self->desc()
+			     '-desc' => $self->desc(),
+                             '-verbose' => $self->verbose
 			     );
    return $out;
 }
@@ -495,33 +512,33 @@ sub translate {
     my($i, $len, $output) = (0,0,'');
     my($codon)   = "";
     my $aa;
-    
+
     ## User can pass in symbol for stop and unknown codons
     unless(defined($stop) and $stop ne '')    { $stop = "*"; }
     unless(defined($unknown) and $unknown ne '') { $unknown = "X"; }
     unless(defined($frame) and $frame ne '') { $frame = 0; }
-    
+
     ## the codon table ID
     unless(defined($tableid) and $tableid ne '')    { $tableid = 1; }
-    
+
     ##Error if monomer is "Amino"
     $self->throw("Can't translate an amino acid sequence.") if
 	($self->alphabet eq 'protein');
-    
+
     ##Error if frame is not 0, 1 or 2
     $self->throw("Valid values for frame are 0, 1, 2, not [$frame].") unless
 	($frame == 0 or $frame == 1 or $frame == 2);
-    
+
     #warns if ID is invalid
     my $codonTable = Bio::Tools::CodonTable->new( -id => $tableid);
-    
+
     my ($seq) = $self->seq();
-    
+
     # deal with frame offset.
     if( $frame ) {
 	$seq = substr ($seq,$frame);
     }
-    
+
     # Translate it
     $output = $codonTable->translate($seq);
     # Use user-input stop/unknown
@@ -555,7 +572,7 @@ sub translate {
 	    }
 	}
     }
-    
+
     my $seqclass;
     if($self->can_call_new()) {
 	$seqclass = ref($self);
@@ -569,10 +586,11 @@ sub translate {
 			      # is there anything wrong with retaining the
 			      # description?
 			      '-desc' => $self->desc(),
-			      '-alphabet' => 'protein'
+			      '-alphabet' => 'protein',
+                              '-verbose' => $self->verbose
 			      );
     return $out;
-    
+
 }
 
 =head2 id
@@ -627,8 +645,7 @@ sub  length {
 
 sub desc {
    my ($self,$value) = @_;
-   $self->warn_not_implemented();
-   return '';
+   $self->throw_not_implemented();
 }
 
 
@@ -645,10 +662,10 @@ sub desc {
 sub is_circular{
     my ($self,$value) = @_;
     if (defined $value) {
-	$self->{'_is_circular'} = 1 if $value;
+	$self->{'_is_circular'} = $value;
     }
     return $self->{'_is_circular'};
-} 
+}
 
 =head1 Private functions
 
