@@ -1,4 +1,4 @@
-# $Id: HTMLResultWriter.pm,v 1.2.2.2 2002/05/30 03:18:56 jason Exp $
+# $Id: HTMLResultWriter.pm,v 1.12 2002/11/23 15:32:24 jason Exp $
 #
 # BioPerl module for Bio::SearchIO::Writer::HTMLResultWriter
 #
@@ -49,7 +49,7 @@ of the bugs and their resolution. Bug reports can be submitted via
 email or the web:
 
   bioperl-bugs@bioperl.org
-  http://bioperl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich
 
@@ -73,12 +73,13 @@ Internal methods are usually preceded with a _
 
 
 package Bio::SearchIO::Writer::HTMLResultWriter;
-use vars qw(@ISA %RemoteURLDefault $MaxDescLen $AlignmentLineWidth);
+use vars qw(@ISA %RemoteURLDefault $MaxDescLen $DATE $AlignmentLineWidth);
 use strict;
 
 # Object preamble - inherits from Bio::Root::RootI
 
 BEGIN {
+    $DATE = localtime(time);
     %RemoteURLDefault = ( 'PROTEIN' => 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=protein&cmd=search&term=%s',			  
 			  'NUCLEOTIDE' => 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=nucleotide&cmd=search&term=%s'
 			  );
@@ -107,10 +108,22 @@ sub new {
   my($class,@args) = @_;
 
   my $self = $class->SUPER::new(@args);
-  my ($p,$n) = $self->_rearrange([qw(PROTEIN_URL 
-				     NUCLEOTIDE_URL)],@args);
+  my ($p,$n,$filters) = $self->_rearrange([qw(PROTEIN_URL 
+					     NUCLEOTIDE_URL 
+					     FILTERS)],@args);
   $self->remote_database_url('p',$p || $RemoteURLDefault{'PROTEIN'});
   $self->remote_database_url('n',$n || $RemoteURLDefault{'NUCLEOTIDE'});
+
+  if( defined $filters ) {
+      if( !ref($filters) =~ /HASH/i ) { 
+	  $self->warn("Did not provide a hashref for the FILTERS option, ignoring.");
+      } else { 
+	  while( my ($type,$code) = each %{$filters} ) {
+	      $self->filter($type,$code);
+	  }
+      }
+  }
+
   return $self;
 }
 
@@ -157,16 +170,31 @@ sub remote_database_url{
 =cut
 
 sub to_string {
-    my ($self,$result) = @_; 
+    my ($self,$result,$num) = @_; 
     return unless defined $result;
+    my ($resultfilter,$hitfilter, $hspfilter) = ( $self->filter('RESULT'),
+						  $self->filter('HIT'),
+						  $self->filter('HSP') );
+    return '' if( defined $resultfilter && ! &{$resultfilter}($result) );    
+
     my $type = ( $result->algorithm =~ /(P|X|Y)$/i ) ? 'PROTEIN' : 'NUCLEOTIDE';
-    my $str = sprintf(
+    my $reference = $result->algorithm_reference || $self->algorithm_reference($result);
+    $reference =~ s/\~/\n/g;
+    my $str;
+    if( $num <= 1 ) { 
+	$str = sprintf(
 qq{<HTML>
-    <HEAD><CENTER><TITLE>Bioperl Reformatted HTML of %s Search output with Bio::Search</TITLE></CENTER></HEAD>
+    <HEAD><CENTER><TITLE>Bioperl Reformatted HTML of %s Search output with Bioperl Bio::SearchIO system</TITLE></CENTER></HEAD>
     <BODY BGCOLOR="WHITE">
+},$result->algorithm);
+
+    }
+    $str .= sprintf(	    
+qq{
     <CENTER><H1>Bioperl Reformatted HTML of %s Search Report<br> for %s</H1></CENTER>
     <hr>
-    <b>Query=</b>%s %s<br><dd>(%d letters)</dd>
+	<pre>%s</pre>
+    <b>Query=</b>%s %s<br><dd>(%s letters)</dd>
     <p>
     <b>Database:</b> %s<br><dd>%d sequences; %d total letters<p></dd>
     <p>
@@ -174,49 +202,67 @@ qq{<HTML>
     <tr><th>Sequences producing significant alignments:</th>
 	<th>Score<br>(bits)</th><th>E<br>value</th></tr>
   }, 
-		      $result->algorithm, $result->algorithm,
-		      $result->query_name(), $result->query_name, 
-		      $result->query_description, $result->query_length, 
-		      $result->database_name(),
-		      $result->database_entries(),$result->database_letters(),
-		      );
+		    $result->algorithm,		      
+		    $result->query_name(), 
+		    $reference,
+		    $result->query_name, 
+		    $result->query_description, $result->query_length, 
+		    $result->database_name(),
+		    $result->database_entries(),$result->database_letters(),
+		    );
     my $hspstr = '<p><p>';
+    if( $result->can('rewind')) {
+        $result->rewind(); # support stream based parsing routines
+    }
+    
     while( my $hit = $result->next_hit ) {
+	next if( $hitfilter && ! &{$hitfilter}($hit) );
 	my $nm = $hit->name();
 	my $id_parser = $self->id_parser;
 	print STDERR "no $nm for name (",$hit->description(), "\n" unless $nm;
 	my ($gi,$acc) = &$id_parser($nm);
-	my $descsub = substr($hit->description,0,$MaxDescLen);
-	$descsub .= " ..." if length($descsub) < length($hit->description);
+	my $p = "%-$MaxDescLen". "s";
+	my $descsub;
+	if( length($hit->description) > ($MaxDescLen - 3) ) {
+	    $descsub = sprintf($p,
+		substr($hit->description,0,$MaxDescLen-3) . "...");
+	} else { 
+	    $descsub = sprintf($p,$hit->description);
+	}
 	
-	$str .= sprintf('<tr><td><a href="%s">%s</a> %s </td><td>%s</td><td><a href="#%s">%s</a></td></tr>'."\n",
-			sprintf($self->remote_database_url($type), 
-				$gi || $acc),
-			$hit->name, $descsub,
-			defined $hit->raw_score ? $hit->raw_score : '?',
+	my $url = length($self->remote_database_url($type)) > 0 ? 
+	    sprintf('<a href="%s">%s</a>',
+		    sprintf($self->remote_database_url($type),$gi || $acc), 
+		    $hit->name()) :  $hit->name();
+
+	my @hsps = $hit->hsps;
+	
+	# failover to first HSP if the data does not contain a 
+	# bitscore/significance value for the Hit (NCBI XML data for one)
+	
+	$str .= sprintf('<tr><td>%s %s</td><td>%s</td><td><a href="#%s">%.2g</a></td></tr>'."\n",
+			$url, $descsub, 
+			($hit->raw_score ? $hit->raw_score : 
+			(defined $hsps[0] ? $hsps[0]->score : ' ')),
 			$acc,
-			defined $hit->significance ? $hit->significance : '?');
+			( $hit->significance ? $hit->significance :
+			 (defined $hsps[0] ? $hsps[0]->evalue : ' ')) 
+			);
 
 	$hspstr .= "<a name=\"$acc\"><pre>\n".
-	    sprintf(">%s %s\n<dd>Length = %d</dd><p>\n\n", $hit->name, 
+	    sprintf("><b>%s</b> %s\n<dd>Length = %d</dd><p>\n\n", $hit->name, 
 			defined $hit->description ? $hit->description : '', 
 		    $hit->length);
 	
-	while( my $hsp = $hit->next_hsp ) {
-	    if( defined $hsp->bits() ) {
-		$hspstr .= sprintf(" Score = %s", $hsp->bits());
-	    }
-	    if( defined $hsp->bits() ) {
-		$hspstr .= sprintf(" bits (%s)", $hsp->score());
-	    }
-	    if( defined $hsp->evalue ) {
-		$hspstr.= sprintf(", Expect = %s", $hsp->evalue);
-	    }
+	foreach my $hsp (@hsps ) {
+	    next if( $hspfilter && ! &{$hspfilter}($hsp) );
+	    $hspstr .= sprintf(" Score = %s bits (%s), Expect = %s",
+			       $hsp->bits, $hsp->score, $hsp->evalue);
 	    if( $hsp->pvalue ) {
 		$hspstr .= ", P = ".$hsp->pvalue;
 	    }
 	    $hspstr .= "<br>\n";
-	    $hspstr .= sprintf(" Identities = %d / %d (%d%%)",
+	    $hspstr .= sprintf(" Identities = %d/%d (%d%%)",
 			         ( $hsp->frac_identical('total') * 
 				   $hsp->length('total')),
 			       $hsp->length('total'),
@@ -227,7 +273,7 @@ qq{<HTML>
 		$hit->algorithm eq 'TBLASTN' ||
 		$hit->algorithm eq 'TFASTN' 
 		) {
-		$hspstr .= sprintf(", Positives = %d / %d (%d%%)",
+		$hspstr .= sprintf(", Positives = %d/%d (%d%%)",
 				   ( $hsp->frac_conserved('total') * 
 				     $hsp->length('total')),
 				   $hsp->length('total'),
@@ -235,7 +281,7 @@ qq{<HTML>
 		
 	    }
 	    if( $hsp->gaps ) {
-		$hspstr .= sprintf(", Gaps = %d / %d (%d%%)",
+		$hspstr .= sprintf(", Gaps = %d/%d (%d%%)",
 				   $hsp->gaps('total'),
 				   $hsp->length('total'),
 				   (100 * $hsp->gaps('total') / 
@@ -257,10 +303,10 @@ qq{<HTML>
 	    
 	    my @hspvals = ( {'name' => 'Query:',
 			     'seq'  => $hsp->query_string,
-			     'start' => $hsp->query->strand > 0 ? $hsp->query->start : $hsp->query->end,
-			     'end'   => $hsp->query->strand > 0 ? $hsp->query->end : $hsp->query->start,
+			     'start' => $hsp->query->strand >= 0 ? $hsp->query->start : $hsp->query->end,
+			     'end'   => $hsp->query->strand >= 0 ? $hsp->query->end : $hsp->query->start,
 			     'index' => 0,
-			     'direction' => $hsp->query->strand
+			     'direction' => $hsp->query->strand || 1
 			     },
 			    { 'name' => ' 'x6,
 			      'seq'  => $hsp->homology_string,
@@ -271,10 +317,10 @@ qq{<HTML>
 			      },
 			    { 'name'  => 'Sbjct:',
 			      'seq'   => $hsp->hit_string,
-			      'start' => $hsp->hit->strand > 0 ? $hsp->hit->start : $hsp->hit->end,
-			      'end'   => $hsp->hit->strand > 0 ? $hsp->hit->end : $hsp->hit->start,
-			      'index' => 0,
-			      'direction' => $hsp->query->strand
+			      'start' => $hsp->hit->strand >= 0 ? $hsp->hit->start : $hsp->hit->end,
+			      'end'   => $hsp->hit->strand >= 0 ? $hsp->hit->end : $hsp->hit->start,
+			      'index' => 0, 
+			      'direction' => $hsp->hit->strand || 1
 			      }
 			    );	    
 	    
@@ -290,7 +336,7 @@ qq{<HTML>
 	    my $count = 0;
 	    while ( $count <= $hsp->length('total') ) {
 		foreach my $v ( @hspvals ) {
-		    my $piece = substr($v->{'seq'}, $v->{'index'},
+		    my $piece = substr($v->{'seq'}, $v->{'index'} + $count,
 				       $AlignmentLineWidth);
 		    my $cp = $piece;
 		    my $plen = scalar ( $cp =~ tr/\-//);
@@ -299,9 +345,15 @@ qq{<HTML>
 			$start = $v->{'start'};
 			# since strand can be + or - use the direction
 			# to signify which whether to add or substract from end
-			$end   = $v->{'start'} + ( ( $v->{'direction'} ) * 
-					   ( $AlignmentLineWidth - $plen ));
-			$v->{'start'} += $AlignmentLineWidth;
+			my $d = ( ( $v->{'direction'} ) * 
+				  ( $AlignmentLineWidth - $plen ));
+			if( length($piece) < $AlignmentLineWidth ) {
+			    $d = (length($piece) - $plen) * $v->{'direction'};
+			}
+			$end   = $v->{'start'} + $d - 1;
+
+
+			$v->{'start'} += $d;
 		    }
 		    $hspstr .= sprintf("%s %-".$numwidth."s %s %s\n",
 				       $v->{'name'},
@@ -311,26 +363,44 @@ qq{<HTML>
 				       );
 		}
 		$count += $AlignmentLineWidth;
-		$hspstr .= '<p>';
+		$hspstr .= "\n\n";
 	    }
 	}
 	$hspstr .= "</pre>\n";
     }
-    $str .= "</table><p>\n".$hspstr;
-
+    $str .= "</table><p>\n".$hspstr."<p><p><hr><h2>Search Parameters</h2><table border=1><tr><th>Parameter</th><th>Value</th>\n";
+    
+    
     foreach my $param ( $result->available_parameters ) {
-	$str .= "$param=". $result->get_parameter($param) ."<br>\n";
+	$str .= "<tr><td>$param</td><td>". $result->get_parameter($param) ."</td></tr>\n";
 	
     }
-    $str .= "<p>";
+    $str .= "</table><p><h2>Search Statistics</h2><table border=1><tr><th>Statistic</th><th>Value</th></tr>\n";
    foreach my $stat ( sort $result->available_statistics ) {
-	$str .= "$stat=". $result->get_statistic($stat). "<br>\n";
+	$str .= "<tr><td>$stat</td><td>". $result->get_statistic($stat). "</td></th>\n";
     }
-    $str .=  $self->footer() . "</BODY>\n</HTML>\n";
+    $str .=  "</table>".$self->footer() . "</BODY>\n</HTML>\n";
     
     return $str;
 }
 
+=head2 end_report
+
+ Title   : end_report
+ Usage   : $self->end_report()
+ Function: The method to call when ending a report, this is
+           mostly for cleanup for formats which require you to 
+           have something at the end of the document (</BODY></HTML>)
+           for HTML
+ Returns : string
+ Args    : none
+
+=cut
+
+sub end_report {
+    return "</BODY>\n</HTML>\n";
+}
+
 # copied from Bio::Index::Fasta
 # useful here as well
 
@@ -397,8 +467,60 @@ sub MAX { $a <=> $b ? $b : $a; }
 
 sub footer { 
     my ($self) = @_;
-    return "<hr><h6>Produced by Bioperl module".ref($self)."</h6>\n"
+    return "<hr><h5>Produced by Bioperl module ".ref($self)." on $DATE</h5>\n"
     
 }
 
+=head2 algorithm_reference
+
+ Title   : algorithm_reference
+ Usage   : my $reference = $writer->algorithm_reference($result);
+ Function: Returns the appropriate Bibliographic reference for the 
+           algorithm format being produced
+ Returns : String
+ Args    : L<Bio::Search::Result::ResultI> to reference
+
+
+=cut
+
+sub algorithm_reference{
+   my ($self,$result) = @_;
+   return '' if( ! defined $result || !ref($result) ||
+		 ! $result->isa('Bio::Search::Result::ResultI')) ;   
+   if( $result->algorithm =~ /BLAST/i ) {
+       my $res = $result->algorithm . ' '. $result->algorithm_version. "\n";
+       if( $result->algorithm_version =~ /WashU/i ) {
+	   return $res .
+"Copyright (C) 1996-2000 Washington University, Saint Louis, Missouri USA.\n
+All Rights Reserved.\n
+\n 
+Reference:  Gish, W. (1996-2000) <a href=\"http://blast.wustl.edu\">http://blast.wustl.edu</a>\n";	   
+       } else {
+	   return $res . 
+"Reference: Altschul, Stephen F., Thomas L. Madden, Alejandro A. Schaffer,\n
+Jinghui Zhang, Zheng Zhang, Webb Miller, and David J. Lipman (1997),\n
+\"Gapped BLAST and PSI-BLAST: a new generation of protein database search\n
+programs\",  Nucleic Acids Res. 25:3389-3402.\n";
+
+       }       
+   } elsif( $result->algorithm =~ /FAST/i ) {
+       return $result->algorithm. " ". $result->algorithm_version . "\n".
+	   "\nReference: Pearson et al, Genomics (1997) 46:24-36\n";
+   } else { 
+       return '';
+   }
+}
+
+=head2 filter
+
+ Title   : filter
+ Usage   : $writer->filter('hsp', \&hsp_filter);
+ Function: Filter out either at HSP,Hit,or Result level
+ Returns : none
+ Args    : string => data type,
+           CODE reference
+
+
+=cut
+
 1;
