@@ -1,4 +1,4 @@
-# $Id: UTR.pm,v 1.2.2.1 2002/05/09 20:51:15 jason Exp $
+# $Id: UTR.pm,v 1.6 2002/10/22 07:45:20 lapp Exp $
 #
 # BioPerl module for Bio::SeqFeature::Gene::UTR
 #
@@ -42,7 +42,7 @@ of the bugs and their resolution. Bug reports can be submitted via
 email or the web:
 
   bioperl-bugs@bioperl.org
-  http://bioperl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - David Block
 
@@ -78,6 +78,29 @@ sub new {
   my($class,@args) = @_;
 
   my $self = $class->SUPER::new(@args);
+  return $self;
+}
+
+=head2 primary_tag
+
+ Title   : primary_tag
+ Usage   : $tag = $feat->primary_tag()
+ Function: Returns the primary tag for a feature,
+           eg 'utr5prime'.  This method insures that 5prime/3prime information
+           is uniformly stored
+ Returns : a string 
+ Args    : none
+
+=cut
+
+sub primary_tag{
+   my ($self,$val) = @_;
+   if( defined $val ) {
+       if ($val =~ /(3|5)/ ) { $val = "utr$1prime" }
+       else { $self->warn("tag should contain indication if this is 3 or 5 prime.  Preferred text is 'utr3prime' or 'utr5prime'.  Using user text of '$val'");}
+   }
+   $self->SUPER::primary_tag($val);
+
 }
 
 1;
