@@ -1,4 +1,4 @@
-# $Id: BPbl2seq.pm,v 1.18 2001/12/14 16:40:18 heikki Exp $
+# $Id: BPbl2seq.pm,v 1.21 2002/10/22 07:38:45 lapp Exp $
 #
 # Bioperl module Bio::Tools::BPbl2seq
 #	based closely on the Bio::Tools::BPlite modules
@@ -42,7 +42,7 @@ alignment using the BLAST algorithm.
 	 $hsp->query->end;
 	 $hsp->sbjct->start;
 	 $hsp->sbjct->end;
-	 $hsp->sbjct->seqname;
+	 $hsp->sbjct->seq_id;
 	 $hsp->sbjct->overlaps($exon);
  }
 
@@ -51,8 +51,7 @@ alignment using the BLAST algorithm.
 BPbl2seq is a package for parsing BLAST bl2seq reports. BLAST bl2seq is a
 program for comparing and aligning two sequences using BLAST.  Although
 the report format is similar to that of a conventional BLAST, there are a
-few differences so that the standard bioperl BLAST parsers Blast.pm and
-BPlite are unable to read bl2seq reports directly.
+few differences so that BPlite is unable to read bl2seq reports directly.
 
 From the user's perspective, one difference between bl2seq and
 other blast reports is that the bl2seq report does not print out the
@@ -100,7 +99,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
  email or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Peter Schattner
 
