@@ -1,5 +1,5 @@
 #------------------------------------------------------------------
-# $Id: RestrictionEnzyme.pm,v 1.19.2.1 2002/04/21 14:30:22 jason Exp $
+# $Id: RestrictionEnzyme.pm,v 1.25 2002/10/28 16:57:04 jason Exp $
 #
 # BioPerl module Bio::Tools::RestrictionEnzyme
 #
@@ -12,7 +12,8 @@
 
 =head1 NAME
 
-Bio::Tools::RestrictionEnzyme - Bioperl object for a restriction endonuclease (cuts DNA at specific locations)
+Bio::Tools::RestrictionEnzyme - Bioperl object for a restriction endonuclease
+(cuts DNA at specific locations)
 
 =head1 SYNOPSIS
 
@@ -31,13 +32,18 @@ Bio::Tools::RestrictionEnzyme - Bioperl object for a restriction endonuclease (c
     $re2 = new Bio::Tools::RestrictionEnzyme(-NAME =>'EcoRV--GAT^ATC', 
 				  	     -MAKE =>'custom');
 
+    ## Get a list of the resulting fragments when a sequence is cut with
+    ## the given enzyme. The method expects a Bio::Seq object.
+
+    @fragments = $re2->cut_seq($seqobj);
+
     ## Get a list of names of all available restriction enzymes 
     ## known to this module.
 
     @all = $re->available_list();
 
     ## Get the names of restriction enzymes that have 6 bp 
-    ## recognition sequences
+    ## recognition sequences.
 
     @sixcutters = $re->available_list(6);
 
@@ -59,14 +65,15 @@ analysis of DNA sequences.
 
 =head2 Considerations
 
-This module is a precursor for a more full featured version that may do such things as
-download data from online databases such as REBase http://www.neb.com/rebase/.
+This module is a precursor for a more full featured version that may do such
+things as download data from online databases such as REBase http://www.neb.com/rebase/.
 Thus, there is currently no functionality for obtaining data about commercial
 availability for a restriction enzyme.
 
-At some point in the future, it may make sense to derive RestrictionEnzymes from
-a class such as Bio::Enzyme or Bio::Prot::Protein (neither of which now exist) 
-so that more data about the enzyme and related information can be easily obtained.
+At some point in the future, it may make sense to derive RestrictionEnzymes
+from a class such as Bio::Enzyme or Bio::Prot::Protein (neither of which now
+exist) so that more data about the enzyme and related information can be 
+easily obtained.
 
 This module is currently in use at 
 
@@ -77,7 +84,8 @@ This module is currently in use at
 
 To digest a sequence on runs of N's in the sequence. Here's what you can do:
 
-    $re_n  = new Bio::Tools::RestrictionEnzyme(-name=>'N--NNNNN', -make=>'custom');
+    $re_n  = new Bio::Tools::RestrictionEnzyme(-name=>'N--NNNNN',
+                                               -make=>'custom');
 
 Specify the number of N's you want to match in the -name parameter. 
 So the above example will recognize and cut at runs of 5 Ns.
@@ -85,9 +93,10 @@ So the above example will recognize and cut at runs of 5 Ns.
 
      -name => 'N--NNNNNNNNNN'
 
-Note that you must use a specific number of N's, you cannot use a regexp to digest at N+ for example,
- because the actual number of N's at each site are not recorded when the sequence is analyzed. 
-So cut_locations( ) wouldn't be correct. 
+Note that you must use a specific number of N's, you cannot use a regexp to
+digest at N+ for example, because the actual number of N's at each site are
+not recorded when the sequence is analyzed. So cut_locations( ) wouldn't be 
+correct. 
 
 =head1 EXAMPLES
 
@@ -95,27 +104,27 @@ See the script examples/restriction.pl in the Bioperl distribution.
 
 =head1 DEPENDENCIES 
 
-Bio::Tools::RestrictionEnzyme.pm is a concrete class that inherits from B<Bio::Root::Root>
-and uses by delegation B<Bio::PrimarySeq>.
+Bio::Tools::RestrictionEnzyme.pm is a concrete class that inherits from 
+B<Bio::Root::Root> and uses by delegation B<Bio::PrimarySeq>.
 
 =head1 FEEDBACK
 
 =head2 Mailing Lists 
 
-User feedback is an integral part of the evolution of this and other Bioperl modules.
-Send your comments and suggestions preferably to one of the Bioperl mailing lists.
-Your participation is much appreciated.
+User feedback is an integral part of the evolution of this and other Bioperl
+modules. Send your comments and suggestions preferably to one of the Bioperl
+mailing lists. Your participation is much appreciated.
 
    bioperl-l@bioperl.org             - General discussion
    http://bioperl.org/MailList.shtml - About the mailing lists
 
 =head2 Reporting Bugs
 
-Report bugs to the Bioperl bug tracking system to help us keep track the bugs and 
-their resolution. Bug reports can be submitted via email or the web:
+Report bugs to the Bioperl bug tracking system to help us keep track the bugs
+and their resolution. Bug reports can be submitted via email or the web:
 
-    bioperl-bugs@bio.perl.org                   
-    http://bio.perl.org/bioperl-bugs/           
+    bioperl-bugs@bio.perl.org
+    http://bugzilla.bioperl.org/
 
 =head1 AUTHOR
 
@@ -169,167 +178,545 @@ use vars qw (@ISA @EXPORT_OK %EXPORT_TAGS $ID $version @RE_available $Revision);
 
 $ID = 'Bio::Tools::RestrictionEnzyme';
 $version = 0.04;
-$Revision = '$Id: RestrictionEnzyme.pm,v 1.19.2.1 2002/04/21 14:30:22 jason Exp $';  #'
+$Revision = '$Id: RestrictionEnzyme.pm,v 1.25 2002/10/28 16:57:04 jason Exp $';  #'
 
-# Generated from REBASE version 802 (strider format), dated Jan 29 98
-# by rebase2perl.pl (JA Feb 98). Merged with previous list by Ewan, Nov 1998
-# Syntax: RE-name => 'SITE CUTS-AT' where SITE and CUTS-AT are separated by a space.
+# Generated from REBASE version 208 (strider format), dated Aug 1 2002
+# using scripts/contributed/rebase2list.pl
+# Syntax: RE-name => 'SITE CUTS-AT' where SITE and CUTS-AT are separated 
+# by a space.
 
 my %RE = (
-          'AatII'   => 'GACGTC 5',
-          'AccI'    => 'GTMKAC 2',
-          'AclI'    => 'AACGTT 2',
-          'AcyI'    => 'GRCGYC 2',
-          'AflII'   => 'CTTAAG 1',
-          'AflIII'  => 'ACRYGT 1',
-          'AgeI'    => 'ACCGGT 1',
-          'AhaIII'  => 'TTTAAA 3',
-	  'AhdI'    => 'GACNNNNNGTC 6',
-          'AluI'    => 'AGCT 2',
-          'AlwNI'   => 'CAGNNNCTG 6',
-          'ApaBI'   => 'GCANNNNNTGC 8',
-          'ApaI'    => 'GGGCCC 5',
-          'ApaLI'   => 'GTGCAC 1',
-          'ApoI'    => 'RAATTY 1',
-          'AscI'    => 'GGCGCGCC 2',
-          'AsuI'    => 'GGNCC 1',
-          'AsuII'   => 'TTCGAA 2',
-          'AvaI'    => 'CYCGRG 1',
-          'AvaII'   => 'GGWCC 1',
-          'AvrII'   => 'CCTAGG 1',
-          'BalI'    => 'TGGCCA 3',
-          'BamHI'   => 'GGATCC 1',
-          'BclI'    => 'TGATCA 1',
-          'BetI'    => 'WCCGGW 1',
-          'BglI'    => 'GCCNNNNNGGC 7',
-          'BglII'   => 'AGATCT 1',
-          'BsaAI'   => 'YACGTR 3',
-          'BsaBI'   => 'GATNNNNATC 5',
-          'BsePI'   => 'GCGCGC 1',
-          'BsiYI'   => 'CCNNNNNNNGG 7',
-          'Bsp1407I'=> 'TGTACA 1',
-          'BspHI'   => 'TCATGA 1',
-          'BspLU11I'=> 'ACATGT 1',
-          'BspMII'  => 'TCCGGA 1',
-          'BstEII'  => 'GGTNACC 1',
-          'BstXI'   => 'CCANNNNNNTGG 8',
-          'Cac8I'   => 'GCNNGC 3',
-          'CauII'   => 'CCSGG 2',
-          'Cfr10I'  => 'RCCGGY 1',
-          'CfrI'    => 'YGGCCR 1',
-          'ClaI'    => 'ATCGAT 2',
-          'CviJI'   => 'RGCY 2',
-          'CviRI'   => 'TGCA 2',
-          'DdeI'    => 'CTNAG 1',
-          'DpnI'    => 'GATC 2',
-	  'DraI'    => 'TTTAAA 3',
-          'DraII'   => 'RGGNCCY 2',
-          'DraIII'  => 'CACNNNGTG 6',
-          'DrdI'    => 'GACNNNNNNGTC 7',
-          'DsaI'    => 'CCRYGG 1',
-          'Eam1105I'=> 'GACNNNNNGTC 6',
-          'Eco47III'=> 'AGCGCT 3',
-          'EcoNI'   => 'CCTNNNNNAGG 5',
-          'EcoRI'   => 'GAATTC 1',
-          'EcoRII'  => 'CCWGG 0',
-          'EcoRV'   => 'GATATC 3',
-          'EspI'    => 'GCTNAGC 2',
-          'Fnu4HI'  => 'GCNGC 2',
-          'FnuDII'  => 'CGCG 2',
-          'FseI'    => 'GGCCGGCC 6',
-          'HaeI'    => 'WGGCCW 3',
-          'HaeII'   => 'RGCGCY 5',
-          'HaeIII'  => 'GGCC 2',
-          'HgiAI'   => 'GWGCWC 5',
-          'HgiCI'   => 'GGYRCC 1',
-          'HgiJII'  => 'GRGCYC 5',
-          'HhaI'    => 'GCGC 3',
-	  'HincII'  => 'GTYRAC 3',
-          'HindII'  => 'GTYRAC 3',
-          'HindIII' => 'AAGCTT 1',
-          'HinfI'   => 'GANTC 1',
-          'HpaI'    => 'GTTAAC 3',
-          'HpaII'   => 'CCGG 1',
-          'KpnI'    => 'GGTACC 5',
-          'MaeI'    => 'CTAG 1',
-          'MaeII'   => 'ACGT 1',
-          'MaeIII'  => 'GTNAC 0',
-          'MboI'    => 'GATC 0',
-          'McrI'    => 'CGRYCG 4',
-          'MfeI'    => 'CAATTG 1',
-          'MluI'    => 'ACGCGT 1',
-          'MseI'    => 'TTAA 1',
-          'MslI'    => 'CAYNNNNRTG 5',
-          'MstI'    => 'TGCGCA 3',
-          'MwoI'    => 'GCNNNNNNNGC 7',
-          'NaeI'    => 'GCCGGC 3',
-          'NarI'    => 'GGCGCC 2',
-          'NcoI'    => 'CCATGG 1',
-          'NdeI'    => 'CATATG 2',
-          'NheI'    => 'GCTAGC 1',
-          'NlaIII'  => 'CATG 4',
-          'NlaIV'   => 'GGNNCC 3',
-          'NotI'    => 'GCGGCCGC 2',
-          'NruI'    => 'TCGCGA 3',
-          'NspBII'  => 'CMGCKG 3',
-          'NspI'    => 'RCATGY 5',
-          'PacI'    => 'TTAATTAA 5',
-          'PflMI'   => 'CCANNNNNTGG 7',
-          'PmaCI'   => 'CACGTG 3',
-          'PmeI'    => 'GTTTAAAC 4',
-          'PpuMI'   => 'RGGWCCY 2',
-          'PshAI'   => 'GACNNNNGTC 5',
-          'PstI'    => 'CTGCAG 5',
-          'PvuI'    => 'CGATCG 4',
-          'PvuII'   => 'CAGCTG 3',
-          'RsaI'    => 'GTAC 2',
-          'RsrII'   => 'CGGWCCG 2',
-          'SacI'    => 'GAGCTC 5',
-          'SacII'   => 'CCGCGG 4',
-	  'Sau96I'  => 'GGNCC 1',
-          'SalI'    => 'GTCGAC 1',
-          'SanDI'   => 'GGGWCCC 2',
-          'SauI'    => 'CCTNAGG 2',
-	  'SbfI'    => 'CCTGCAGG 6',
-          'ScaI'    => 'AGTACT 3',
-          'ScrFI'   => 'CCNGG 2',
-          'SduI'    => 'GDGCHC 5',
-          'SecI'    => 'CCNNGG 1',
-          'SexAI'   => 'ACCWGGT 1',
-          'SfeI'    => 'CTRYAG 1',
-          'SfiI'    => 'GGCCNNNNNGGCC 8',
-          'SgfI'    => 'GCGATCGC 5',
-          'SgrAI'   => 'CRCCGGYG 2',
-          'SmaI'    => 'CCCGGG 3',
-          'SmlI'    => 'CTYRAG 1',
-          'SnaBI'   => 'TACGTA 3',
-          'SpeI'    => 'ACTAGT 1',
-          'SphI'    => 'GCATGC 5',
-          'SplI'    => 'CGTACG 1',
-          'SrfI'    => 'GCCCGGGC 4',
-          'Sse8387I'=> 'CCTGCAGG 6',
-          'Sse8647I'=> 'AGGWCCT 2',
-          'SspI'    => 'AATATT 3',
-          'StuI'    => 'AGGCCT 3',
-          'StyI'    => 'CCWWGG 1',
-          'SwaI'    => 'ATTTAAAT 4',
-          'TaqI'    => 'TCGA 1',
-          'TatI'    => 'WGTACW 1',
-          'TfiI'    => 'GAWTC 1',
-          'TseI'    => 'GCWGC 1',
-          'Tsp45I'  => 'GTSAC 0',
-          'Tsp4CI'  => 'ACNGT 3',
-          'TspEI'   => 'AATT 0',
-          'TspRI'   => 'CASTGNN 7',
-          'Tth111I' => 'GACNNNGTC 4',
-          'VspI'    => 'ATTAAT 2',
-          'XbaI'    => 'TCTAGA 1',
-          'XcmI'    => 'CCANNNNNNNNNTGG 8',
-          'XhoI'    => 'CTCGAG 1',
-          'XhoII'   => 'RGATCY 1',
-          'XmaIII'  => 'CGGCCG 1',
-          'XmnI'    => 'GAANNNNTTC 5',
-         );
+ 'AasI'	=> 'GACNNNNNNGTC 7',
+ 'AatI'	=> 'AGGCCT 3',
+ 'AatII'	=> 'GACGTC 5',
+ 'AauI'	=> 'TGTACA 1',
+ 'AccI'	=> 'GTMKAC 2',
+ 'AccII'	=> 'CGCG 2',
+ 'AccIII'	=> 'TCCGGA 1',
+ 'Acc16I'	=> 'TGCGCA 3',
+ 'Acc65I'	=> 'GGTACC 1',
+ 'Acc113I'	=> 'AGTACT 3',
+ 'AccB1I'	=> 'GGYRCC 1',
+ 'AccB7I'	=> 'CCANNNNNTGG 7',
+ 'AclI'	=> 'AACGTT 2',
+ 'AcsI'	=> 'RAATTY 1',
+ 'AcvI'	=> 'CACGTG 3',
+ 'AcyI'	=> 'GRCGYC 2',
+ 'AdeI'	=> 'CACNNNGTG 6',
+ 'AfaI'	=> 'GTAC 2',
+ 'AfeI'	=> 'AGCGCT 3',
+ 'AflI'	=> 'GGWCC 1',
+ 'AflII'	=> 'CTTAAG 1',
+ 'AflIII'	=> 'ACRYGT 1',
+ 'AgeI'	=> 'ACCGGT 1',
+ 'AhaIII'	=> 'TTTAAA 3',
+ 'AhdI'	=> 'GACNNNNNGTC 6',
+ 'AhlI'	=> 'ACTAGT 1',
+ 'AleI'	=> 'CACNNNNGTG 5',
+ 'AluI'	=> 'AGCT 2',
+ 'Alw21I'	=> 'GWGCWC 5',
+ 'Alw44I'	=> 'GTGCAC 1',
+ 'AlwNI'	=> 'CAGNNNCTG 6',
+ 'Ama87I'	=> 'CYCGRG 1',
+ 'AocI'	=> 'CCTNAGG 2',
+ 'Aor51HI'	=> 'AGCGCT 3',
+ 'ApaI'	=> 'GGGCCC 5',
+ 'ApaBI'	=> 'GCANNNNNTGC 8',
+ 'ApaLI'	=> 'GTGCAC 1',
+ 'ApoI'	=> 'RAATTY 1',
+ 'AscI'	=> 'GGCGCGCC 2',
+ 'AseI'	=> 'ATTAAT 2',
+ 'AsiAI'	=> 'ACCGGT 1',
+ 'AsiSI'	=> 'GCGATCGC 5',
+ 'AsnI'	=> 'ATTAAT 2',
+ 'AspI'	=> 'GACNNNGTC 4',
+ 'Asp700I'	=> 'GAANNNNTTC 5',
+ 'Asp718I'	=> 'GGTACC 1',
+ 'AspEI'	=> 'GACNNNNNGTC 6',
+ 'AspHI'	=> 'GWGCWC 5',
+ 'AspLEI'	=> 'GCGC 3',
+ 'AspS9I'	=> 'GGNCC 1',
+ 'AsuI'	=> 'GGNCC 1',
+ 'AsuII'	=> 'TTCGAA 2',
+ 'AsuC2I'	=> 'CCSGG 2',
+ 'AsuNHI'	=> 'GCTAGC 1',
+ 'AvaI'	=> 'CYCGRG 1',
+ 'AvaII'	=> 'GGWCC 1',
+ 'AviII'	=> 'TGCGCA 3',
+ 'AvrII'	=> 'CCTAGG 1',
+ 'AxyI'	=> 'CCTNAGG 2',
+ 'BalI'	=> 'TGGCCA 3',
+ 'BamHI'	=> 'GGATCC 1',
+ 'BanI'	=> 'GGYRCC 1',
+ 'BanII'	=> 'GRGCYC 5',
+ 'BanIII'	=> 'ATCGAT 2',
+ 'BbeI'	=> 'GGCGCC 5',
+ 'BbrPI'	=> 'CACGTG 3',
+ 'BbuI'	=> 'GCATGC 5',
+ 'Bbv12I'	=> 'GWGCWC 5',
+ 'BclI'	=> 'TGATCA 1',
+ 'BcnI'	=> 'CCSGG 2',
+ 'BcoI'	=> 'CYCGRG 1',
+ 'BcuI'	=> 'ACTAGT 1',
+ 'BetI'	=> 'WCCGGW 1',
+ 'BfaI'	=> 'CTAG 1',
+ 'BfmI'	=> 'CTRYAG 1',
+ 'BfrI'	=> 'CTTAAG 1',
+ 'BfrBI'	=> 'ATGCAT 3',
+ 'BfuCI'	=> 'GATC 0',
+ 'BglI'	=> 'GCCNNNNNGGC 7',
+ 'BglII'	=> 'AGATCT 1',
+ 'BlnI'	=> 'CCTAGG 1',
+ 'BloHII'	=> 'CTGCAG 5',
+ 'BlpI'	=> 'GCTNAGC 2',
+ 'Bme18I'	=> 'GGWCC 1',
+ 'Bme1390I'	=> 'CCNGG 2',
+ 'Bme1580I'	=> 'GKGCMC 5',
+ 'BmtI'	=> 'GCTAGC 5',
+ 'BmyI'	=> 'GDGCHC 5',
+ 'BoxI'	=> 'GACNNNNGTC 5',
+ 'Bpu14I'	=> 'TTCGAA 2',
+ 'Bpu1102I'	=> 'GCTNAGC 2',
+ 'Bsa29I'	=> 'ATCGAT 2',
+ 'BsaAI'	=> 'YACGTR 3',
+ 'BsaBI'	=> 'GATNNNNATC 5',
+ 'BsaHI'	=> 'GRCGYC 2',
+ 'BsaJI'	=> 'CCNNGG 1',
+ 'BsaOI'	=> 'CGRYCG 4',
+ 'BsaWI'	=> 'WCCGGW 1',
+ 'BscI'	=> 'ATCGAT 2',
+ 'Bsc4I'	=> 'CCNNNNNNNGG 7',
+ 'BscBI'	=> 'GGNNCC 3',
+ 'BscFI'	=> 'GATC 0',
+ 'Bse8I'	=> 'GATNNNNATC 5',
+ 'Bse21I'	=> 'CCTNAGG 2',
+ 'Bse118I'	=> 'RCCGGY 1',
+ 'BseAI'	=> 'TCCGGA 1',
+ 'BseBI'	=> 'CCWGG 2',
+ 'BseCI'	=> 'ATCGAT 2',
+ 'BseDI'	=> 'CCNNGG 1',
+ 'BseJI'	=> 'GATNNNNATC 5',
+ 'BseLI'	=> 'CCNNNNNNNGG 7',
+ 'BsePI'	=> 'GCGCGC 1',
+ 'BseSI'	=> 'GKGCMC 5',
+ 'BseX3I'	=> 'CGGCCG 1',
+ 'BshI'	=> 'GGCC 2',
+ 'Bsh1236I'	=> 'CGCG 2',
+ 'Bsh1285I'	=> 'CGRYCG 4',
+ 'BshFI'	=> 'GGCC 2',
+ 'BshNI'	=> 'GGYRCC 1',
+ 'BshTI'	=> 'ACCGGT 1',
+ 'BsiBI'	=> 'GATNNNNATC 5',
+ 'BsiCI'	=> 'TTCGAA 2',
+ 'BsiEI'	=> 'CGRYCG 4',
+ 'BsiHKAI'	=> 'GWGCWC 5',
+ 'BsiHKCI'	=> 'CYCGRG 1',
+ 'BsiLI'	=> 'CCWGG 2',
+ 'BsiMI'	=> 'TCCGGA 1',
+ 'BsiQI'	=> 'TGATCA 1',
+ 'BsiSI'	=> 'CCGG 1',
+ 'BsiWI'	=> 'CGTACG 1',
+ 'BsiXI'	=> 'ATCGAT 2',
+ 'BsiYI'	=> 'CCNNNNNNNGG 7',
+ 'BsiZI'	=> 'GGNCC 1',
+ 'BslI'	=> 'CCNNNNNNNGG 7',
+ 'BsoBI'	=> 'CYCGRG 1',
+ 'Bsp13I'	=> 'TCCGGA 1',
+ 'Bsp19I'	=> 'CCATGG 1',
+ 'Bsp68I'	=> 'TCGCGA 3',
+ 'Bsp106I'	=> 'ATCGAT 2',
+ 'Bsp119I'	=> 'TTCGAA 2',
+ 'Bsp120I'	=> 'GGGCCC 1',
+ 'Bsp143I'	=> 'GATC 0',
+ 'Bsp143II'	=> 'RGCGCY 5',
+ 'Bsp1286I'	=> 'GDGCHC 5',
+ 'Bsp1407I'	=> 'TGTACA 1',
+ 'Bsp1720I'	=> 'GCTNAGC 2',
+ 'BspA2I'	=> 'CCTAGG 1',
+ 'BspCI'	=> 'CGATCG 4',
+ 'BspDI'	=> 'ATCGAT 2',
+ 'BspEI'	=> 'TCCGGA 1',
+ 'BspHI'	=> 'TCATGA 1',
+ 'BspLI'	=> 'GGNNCC 3',
+ 'BspLU11I'	=> 'ACATGT 1',
+ 'BspMII'	=> 'TCCGGA 1',
+ 'BspTI'	=> 'CTTAAG 1',
+ 'BspT104I'	=> 'TTCGAA 2',
+ 'BspT107I'	=> 'GGYRCC 1',
+ 'BspXI'	=> 'ATCGAT 2',
+ 'BsrBRI'	=> 'GATNNNNATC 5',
+ 'BsrFI'	=> 'RCCGGY 1',
+ 'BsrGI'	=> 'TGTACA 1',
+ 'BssAI'	=> 'RCCGGY 1',
+ 'BssECI'	=> 'CCNNGG 1',
+ 'BssHI'	=> 'CTCGAG 1',
+ 'BssHII'	=> 'GCGCGC 1',
+ 'BssKI'	=> 'CCNGG 0',
+ 'BssNAI'	=> 'GTATAC 3',
+ 'BssT1I'	=> 'CCWWGG 1',
+ 'Bst98I'	=> 'CTTAAG 1',
+ 'Bst1107I'	=> 'GTATAC 3',
+ 'BstACI'	=> 'GRCGYC 2',
+ 'BstAPI'	=> 'GCANNNNNTGC 7',
+ 'BstBI'	=> 'TTCGAA 2',
+ 'BstBAI'	=> 'YACGTR 3',
+ 'Bst4CI'	=> 'ACNGT 3',
+ 'BstC8I'	=> 'GCNNGC 3',
+ 'BstDEI'	=> 'CTNAG 1',
+ 'BstDSI'	=> 'CCRYGG 1',
+ 'BstEII'	=> 'GGTNACC 1',
+ 'BstENI'	=> 'CCTNNNNNAGG 5',
+ 'BstENII'	=> 'GATC 0',
+ 'BstFNI'	=> 'CGCG 2',
+ 'BstH2I'	=> 'RGCGCY 5',
+ 'BstHHI'	=> 'GCGC 3',
+ 'BstHPI'	=> 'GTTAAC 3',
+ 'BstKTI'	=> 'GATC 3',
+ 'BstMAI'	=> 'CTGCAG 5',
+ 'BstMCI'	=> 'CGRYCG 4',
+ 'BstMWI'	=> 'GCNNNNNNNGC 7',
+ 'BstNI'	=> 'CCWGG 2',
+ 'BstNSI'	=> 'RCATGY 5',
+ 'BstOI'	=> 'CCWGG 2',
+ 'BstPI'	=> 'GGTNACC 1',
+ 'BstPAI'	=> 'GACNNNNGTC 5',
+ 'BstSCI'	=> 'CCNGG 0',
+ 'BstSFI'	=> 'CTRYAG 1',
+ 'BstSNI'	=> 'TACGTA 3',
+ 'BstUI'	=> 'CGCG 2',
+ 'Bst2UI'	=> 'CCWGG 2',
+ 'BstXI'	=> 'CCANNNNNNTGG 8',
+ 'BstX2I'	=> 'RGATCY 1',
+ 'BstYI'	=> 'RGATCY 1',
+ 'BstZI'	=> 'CGGCCG 1',
+ 'BstZ17I'	=> 'GTATAC 3',
+ 'Bsu15I'	=> 'ATCGAT 2',
+ 'Bsu36I'	=> 'CCTNAGG 2',
+ 'BsuRI'	=> 'GGCC 2',
+ 'BsuTUI'	=> 'ATCGAT 2',
+ 'BtgI'	=> 'CCRYGG 1',
+ 'BthCI'	=> 'GCNGC 4',
+ 'Cac8I'	=> 'GCNNGC 3',
+ 'CaiI'	=> 'CAGNNNCTG 6',
+ 'CauII'	=> 'CCSGG 2',
+ 'CciNI'	=> 'GCGGCCGC 2',
+ 'CelII'	=> 'GCTNAGC 2',
+ 'CfoI'	=> 'GCGC 3',
+ 'CfrI'	=> 'YGGCCR 1',
+ 'Cfr9I'	=> 'CCCGGG 1',
+ 'Cfr10I'	=> 'RCCGGY 1',
+ 'Cfr13I'	=> 'GGNCC 1',
+ 'Cfr42I'	=> 'CCGCGG 4',
+ 'ChaI'	=> 'GATC 4',
+ 'ClaI'	=> 'ATCGAT 2',
+ 'CpoI'	=> 'CGGWCCG 2',
+ 'CspI'	=> 'CGGWCCG 2',
+ 'Csp6I'	=> 'GTAC 1',
+ 'Csp45I'	=> 'TTCGAA 2',
+ 'CspAI'	=> 'ACCGGT 1',
+ 'CviAII'	=> 'CATG 1',
+ 'CviJI'	=> 'RGCY 2',
+ 'CviRI'	=> 'TGCA 2',
+ 'CviTI'	=> 'RGCY 2',
+ 'CvnI'	=> 'CCTNAGG 2',
+ 'DdeI'	=> 'CTNAG 1',
+ 'DpnI'	=> 'GATC 2',
+ 'DpnII'	=> 'GATC 0',
+ 'DraI'	=> 'TTTAAA 3',
+ 'DraII'	=> 'RGGNCCY 2',
+ 'DraIII'	=> 'CACNNNGTG 6',
+ 'DrdI'	=> 'GACNNNNNNGTC 7',
+ 'DsaI'	=> 'CCRYGG 1',
+ 'DseDI'	=> 'GACNNNNNNGTC 7',
+ 'EaeI'	=> 'YGGCCR 1',
+ 'EagI'	=> 'CGGCCG 1',
+ 'Eam1105I'	=> 'GACNNNNNGTC 6',
+ 'Ecl136II'	=> 'GAGCTC 3',
+ 'EclHKI'	=> 'GACNNNNNGTC 6',
+ 'EclXI'	=> 'CGGCCG 1',
+ 'Eco24I'	=> 'GRGCYC 5',
+ 'Eco32I'	=> 'GATATC 3',
+ 'Eco47I'	=> 'GGWCC 1',
+ 'Eco47III'	=> 'AGCGCT 3',
+ 'Eco52I'	=> 'CGGCCG 1',
+ 'Eco72I'	=> 'CACGTG 3',
+ 'Eco81I'	=> 'CCTNAGG 2',
+ 'Eco88I'	=> 'CYCGRG 1',
+ 'Eco91I'	=> 'GGTNACC 1',
+ 'Eco105I'	=> 'TACGTA 3',
+ 'Eco130I'	=> 'CCWWGG 1',
+ 'Eco147I'	=> 'AGGCCT 3',
+ 'EcoHI'	=> 'CCSGG 0',
+ 'EcoICRI'	=> 'GAGCTC 3',
+ 'EcoNI'	=> 'CCTNNNNNAGG 5',
+ 'EcoO65I'	=> 'GGTNACC 1',
+ 'EcoO109I'	=> 'RGGNCCY 2',
+ 'EcoRI'	=> 'GAATTC 1',
+ 'EcoRII'	=> 'CCWGG 0',
+ 'EcoRV'	=> 'GATATC 3',
+ 'EcoT14I'	=> 'CCWWGG 1',
+ 'EcoT22I'	=> 'ATGCAT 5',
+ 'EcoT38I'	=> 'GRGCYC 5',
+ 'EgeI'	=> 'GGCGCC 3',
+ 'EheI'	=> 'GGCGCC 3',
+ 'ErhI'	=> 'CCWWGG 1',
+ 'EsaBC3I'	=> 'TCGA 2',
+ 'EspI'	=> 'GCTNAGC 2',
+ 'FatI'	=> 'CATG 0',
+ 'FauNDI'	=> 'CATATG 2',
+ 'FbaI'	=> 'TGATCA 1',
+ 'FblI'	=> 'GTMKAC 2',
+ 'FmuI'	=> 'GGNCC 4',
+ 'FnuDII'	=> 'CGCG 2',
+ 'Fnu4HI'	=> 'GCNGC 2',
+ 'FriOI'	=> 'GRGCYC 5',
+ 'FseI'	=> 'GGCCGGCC 6',
+ 'FspI'	=> 'TGCGCA 3',
+ 'FspAI'	=> 'RTGCGCAY 4',
+ 'Fsp4HI'	=> 'GCNGC 2',
+ 'FunI'	=> 'AGCGCT 3',
+ 'FunII'	=> 'GAATTC 1',
+ 'HaeI'	=> 'WGGCCW 3',
+ 'HaeII'	=> 'RGCGCY 5',
+ 'HaeIII'	=> 'GGCC 2',
+ 'HapII'	=> 'CCGG 1',
+ 'HgiAI'	=> 'GWGCWC 5',
+ 'HgiCI'	=> 'GGYRCC 1',
+ 'HgiJII'	=> 'GRGCYC 5',
+ 'HhaI'	=> 'GCGC 3',
+ 'Hin1I'	=> 'GRCGYC 2',
+ 'Hin6I'	=> 'GCGC 1',
+ 'HinP1I'	=> 'GCGC 1',
+ 'HincII'	=> 'GTYRAC 3',
+ 'HindII'	=> 'GTYRAC 3',
+ 'HindIII'	=> 'AAGCTT 1',
+ 'HinfI'	=> 'GANTC 1',
+ 'HpaI'	=> 'GTTAAC 3',
+ 'HpaII'	=> 'CCGG 1',
+ 'Hpy8I'	=> 'GTNNAC 3',
+ 'Hpy99I'	=> 'CGWCG 5',
+ 'Hpy178III'	=> 'TCNNGA 2',
+ 'Hpy188I'	=> 'TCNGA 3',
+ 'Hpy188III'	=> 'TCNNGA 2',
+ 'HpyCH4I'	=> 'CATG 3',
+ 'HpyCH4III'	=> 'ACNGT 3',
+ 'HpyCH4IV'	=> 'ACGT 1',
+ 'HpyCH4V'	=> 'TGCA 2',
+ 'HpyF10VI'	=> 'GCNNNNNNNGC 8',
+ 'Hsp92I'	=> 'GRCGYC 2',
+ 'Hsp92II'	=> 'CATG 4',
+ 'HspAI'	=> 'GCGC 1',
+ 'ItaI'	=> 'GCNGC 2',
+ 'KasI'	=> 'GGCGCC 1',
+ 'KpnI'	=> 'GGTACC 5',
+ 'Kpn2I'	=> 'TCCGGA 1',
+ 'KspI'	=> 'CCGCGG 4',
+ 'Ksp22I'	=> 'TGATCA 1',
+ 'KspAI'	=> 'GTTAAC 3',
+ 'Kzo9I'	=> 'GATC 0',
+ 'LpnI'	=> 'RGCGCY 3',
+ 'LspI'	=> 'TTCGAA 2',
+ 'MabI'	=> 'ACCWGGT 1',
+ 'MaeI'	=> 'CTAG 1',
+ 'MaeII'	=> 'ACGT 1',
+ 'MaeIII'	=> 'GTNAC 0',
+ 'MamI'	=> 'GATNNNNATC 5',
+ 'MboI'	=> 'GATC 0',
+ 'McrI'	=> 'CGRYCG 4',
+ 'MfeI'	=> 'CAATTG 1',
+ 'MflI'	=> 'RGATCY 1',
+ 'MhlI'	=> 'GDGCHC 5',
+ 'MlsI'	=> 'TGGCCA 3',
+ 'MluI'	=> 'ACGCGT 1',
+ 'MluNI'	=> 'TGGCCA 3',
+ 'Mly113I'	=> 'GGCGCC 2',
+ 'Mph1103I'	=> 'ATGCAT 5',
+ 'MroI'	=> 'TCCGGA 1',
+ 'MroNI'	=> 'GCCGGC 1',
+ 'MroXI'	=> 'GAANNNNTTC 5',
+ 'MscI'	=> 'TGGCCA 3',
+ 'MseI'	=> 'TTAA 1',
+ 'MslI'	=> 'CAYNNNNRTG 5',
+ 'MspI'	=> 'CCGG 1',
+ 'Msp20I'	=> 'TGGCCA 3',
+ 'MspA1I'	=> 'CMGCKG 3',
+ 'MspCI'	=> 'CTTAAG 1',
+ 'MspR9I'	=> 'CCNGG 2',
+ 'MssI'	=> 'GTTTAAAC 4',
+ 'MstI'	=> 'TGCGCA 3',
+ 'MunI'	=> 'CAATTG 1',
+ 'MvaI'	=> 'CCWGG 2',
+ 'MvnI'	=> 'CGCG 2',
+ 'MwoI'	=> 'GCNNNNNNNGC 7',
+ 'NaeI'	=> 'GCCGGC 3',
+ 'NarI'	=> 'GGCGCC 2',
+ 'NciI'	=> 'CCSGG 2',
+ 'NcoI'	=> 'CCATGG 1',
+ 'NdeI'	=> 'CATATG 2',
+ 'NdeII'	=> 'GATC 0',
+ 'NgoAIV'	=> 'GCCGGC 1',
+ 'NgoMIV'	=> 'GCCGGC 1',
+ 'NheI'	=> 'GCTAGC 1',
+ 'NlaIII'	=> 'CATG 4',
+ 'NlaIV'	=> 'GGNNCC 3',
+ 'Nli3877I'	=> 'CYCGRG 5',
+ 'NmuCI'	=> 'GTSAC 0',
+ 'NotI'	=> 'GCGGCCGC 2',
+ 'NruI'	=> 'TCGCGA 3',
+ 'NruGI'	=> 'GACNNNNNGTC 6',
+ 'NsbI'	=> 'TGCGCA 3',
+ 'NsiI'	=> 'ATGCAT 5',
+ 'NspI'	=> 'RCATGY 5',
+ 'NspIII'	=> 'CYCGRG 1',
+ 'NspV'	=> 'TTCGAA 2',
+ 'NspBII'	=> 'CMGCKG 3',
+ 'OliI'	=> 'CACNNNNGTG 5',
+ 'PacI'	=> 'TTAATTAA 5',
+ 'PaeI'	=> 'GCATGC 5',
+ 'PaeR7I'	=> 'CTCGAG 1',
+ 'PagI'	=> 'TCATGA 1',
+ 'PalI'	=> 'GGCC 2',
+ 'PauI'	=> 'GCGCGC 1',
+ 'PceI'	=> 'AGGCCT 3',
+ 'PciI'	=> 'ACATGT 1',
+ 'PdiI'	=> 'GCCGGC 3',
+ 'PdmI'	=> 'GAANNNNTTC 5',
+ 'Pfl23II'	=> 'CGTACG 1',
+ 'PflBI'	=> 'CCANNNNNTGG 7',
+ 'PflFI'	=> 'GACNNNGTC 4',
+ 'PflMI'	=> 'CCANNNNNTGG 7',
+ 'PfoI'	=> 'TCCNGGA 1',
+ 'PinAI'	=> 'ACCGGT 1',
+ 'Ple19I'	=> 'CGATCG 4',
+ 'PmaCI'	=> 'CACGTG 3',
+ 'PmeI'	=> 'GTTTAAAC 4',
+ 'PmlI'	=> 'CACGTG 3',
+ 'Ppu10I'	=> 'ATGCAT 1',
+ 'PpuMI'	=> 'RGGWCCY 2',
+ 'PpuXI'	=> 'RGGWCCY 2',
+ 'PshAI'	=> 'GACNNNNGTC 5',
+ 'PshBI'	=> 'ATTAAT 2',
+ 'PsiI'	=> 'TTATAA 3',
+ 'Psp03I'	=> 'GGWCC 4',
+ 'Psp5II'	=> 'RGGWCCY 2',
+ 'Psp6I'	=> 'CCWGG 0',
+ 'Psp1406I'	=> 'AACGTT 2',
+ 'PspAI'	=> 'CCCGGG 1',
+ 'Psp124BI'	=> 'GAGCTC 5',
+ 'PspEI'	=> 'GGTNACC 1',
+ 'PspGI'	=> 'CCWGG 0',
+ 'PspLI'	=> 'CGTACG 1',
+ 'PspN4I'	=> 'GGNNCC 3',
+ 'PspOMI'	=> 'GGGCCC 1',
+ 'PspPI'	=> 'GGNCC 1',
+ 'PspPPI'	=> 'RGGWCCY 2',
+ 'PssI'	=> 'RGGNCCY 5',
+ 'PstI'	=> 'CTGCAG 5',
+ 'PsuI'	=> 'RGATCY 1',
+ 'PsyI'	=> 'GACNNNGTC 4',
+ 'PvuI'	=> 'CGATCG 4',
+ 'PvuII'	=> 'CAGCTG 3',
+ 'RcaI'	=> 'TCATGA 1',
+ 'RsaI'	=> 'GTAC 2',
+ 'RsrII'	=> 'CGGWCCG 2',
+ 'Rsr2I'	=> 'CGGWCCG 2',
+ 'SacI'	=> 'GAGCTC 5',
+ 'SacII'	=> 'CCGCGG 4',
+ 'SalI'	=> 'GTCGAC 1',
+ 'SanDI'	=> 'GGGWCCC 2',
+ 'SatI'	=> 'GCNGC 2',
+ 'SauI'	=> 'CCTNAGG 2',
+ 'Sau96I'	=> 'GGNCC 1',
+ 'Sau3AI'	=> 'GATC 0',
+ 'SbfI'	=> 'CCTGCAGG 6',
+ 'ScaI'	=> 'AGTACT 3',
+ 'SciI'	=> 'CTCGAG 3',
+ 'ScrFI'	=> 'CCNGG 2',
+ 'SdaI'	=> 'CCTGCAGG 6',
+ 'SduI'	=> 'GDGCHC 5',
+ 'SecI'	=> 'CCNNGG 1',
+ 'SelI'	=> 'CGCG 0',
+ 'SexAI'	=> 'ACCWGGT 1',
+ 'SfcI'	=> 'CTRYAG 1',
+ 'SfeI'	=> 'CTRYAG 1',
+ 'SfiI'	=> 'GGCCNNNNNGGCC 8',
+ 'SfoI'	=> 'GGCGCC 3',
+ 'Sfr274I'	=> 'CTCGAG 1',
+ 'Sfr303I'	=> 'CCGCGG 4',
+ 'SfuI'	=> 'TTCGAA 2',
+ 'SgfI'	=> 'GCGATCGC 5',
+ 'SgrAI'	=> 'CRCCGGYG 2',
+ 'SgrBI'	=> 'CCGCGG 4',
+ 'SinI'	=> 'GGWCC 1',
+ 'SlaI'	=> 'CTCGAG 1',
+ 'SmaI'	=> 'CCCGGG 3',
+ 'SmiI'	=> 'ATTTAAAT 4',
+ 'SmiMI'	=> 'CAYNNNNRTG 5',
+ 'SmlI'	=> 'CTYRAG 1',
+ 'SnaBI'	=> 'TACGTA 3',
+ 'SpaHI'	=> 'GCATGC 5',
+ 'SpeI'	=> 'ACTAGT 1',
+ 'SphI'	=> 'GCATGC 5',
+ 'SplI'	=> 'CGTACG 1',
+ 'SrfI'	=> 'GCCCGGGC 4',
+ 'Sse9I'	=> 'AATT 0',
+ 'Sse232I'	=> 'CGCCGGCG 2',
+ 'Sse8387I'	=> 'CCTGCAGG 6',
+ 'Sse8647I'	=> 'AGGWCCT 2',
+ 'SseBI'	=> 'AGGCCT 3',
+ 'SspI'	=> 'AATATT 3',
+ 'SspBI'	=> 'TGTACA 1',
+ 'SstI'	=> 'GAGCTC 5',
+ 'SstII'	=> 'CCGCGG 4',
+ 'StuI'	=> 'AGGCCT 3',
+ 'StyI'	=> 'CCWWGG 1',
+ 'SunI'	=> 'CGTACG 1',
+ 'SwaI'	=> 'ATTTAAAT 4',
+ 'TaaI'	=> 'ACNGT 3',
+ 'TaiI'	=> 'ACGT 4',
+ 'TaqI'	=> 'TCGA 1',
+ 'TasI'	=> 'AATT 0',
+ 'TatI'	=> 'WGTACW 1',
+ 'TauI'	=> 'GCSGC 4',
+ 'TelI'	=> 'GACNNNGTC 4',
+ 'TfiI'	=> 'GAWTC 1',
+ 'ThaI'	=> 'CGCG 2',
+ 'TliI'	=> 'CTCGAG 1',
+ 'Tru1I'	=> 'TTAA 1',
+ 'Tru9I'	=> 'TTAA 1',
+ 'TscI'	=> 'ACGT 4',
+ 'TseI'	=> 'GCWGC 1',
+ 'Tsp45I'	=> 'GTSAC 0',
+ 'Tsp509I'	=> 'AATT 0',
+ 'Tsp4CI'	=> 'ACNGT 3',
+ 'TspEI'	=> 'AATT 0',
+ 'Tth111I'	=> 'GACNNNGTC 4',
+ 'TthHB8I'	=> 'TCGA 1',
+ 'UnbI'	=> 'GGNCC 0',
+ 'Van91I'	=> 'CCANNNNNTGG 7',
+ 'Vha464I'	=> 'CTTAAG 1',
+ 'VneI'	=> 'GTGCAC 1',
+ 'VpaK11AI'	=> 'GGWCC 0',
+ 'VpaK11BI'	=> 'GGWCC 1',
+ 'VspI'	=> 'ATTAAT 2',
+ 'XagI'	=> 'CCTNNNNNAGG 5',
+ 'XapI'	=> 'RAATTY 1',
+ 'XbaI'	=> 'TCTAGA 1',
+ 'XceI'	=> 'RCATGY 5',
+ 'XcmI'	=> 'CCANNNNNNNNNTGG 8',
+ 'XhoI'	=> 'CTCGAG 1',
+ 'XhoII'	=> 'RGATCY 1',
+ 'XmaI'	=> 'CCCGGG 1',
+ 'XmaIII'	=> 'CGGCCG 1',
+ 'XmaCI'	=> 'CCCGGG 1',
+ 'XmaJI'	=> 'CCTAGG 1',
+ 'XmiI'	=> 'GTMKAC 2',
+ 'XmnI'	=> 'GAANNNNTTC 5',
+ 'XspI'	=> 'CTAG 1',
+ 'ZhoI'	=> 'ATCGAT 2',
+ 'ZraI'	=> 'GACGTC 3',
+ 'Zsp2I'	=> 'ATGCAT 5',
+);
 
 @RE_available = sort keys %RE;
 
@@ -352,7 +739,7 @@ See Also   : L<_make_custom>(), L<_make_standard>(), B<Bio::PrimarySeq.pm::_init
 sub new {
 #---------------
     my($class, @args) = @_;
-    
+
     my $self = $class->SUPER::new(@args);
     my ($name,$make) = $self->_rearrange([qw(NAME MAKE)],@args);
     $name && $self->name($name);
@@ -399,7 +786,7 @@ sub _make_standard {
     my @data = split( ' ', $RE{$name});
     my (%dat);
     $dat{-SEQ} = $data[0];
-    $dat{-NAME} = $dat{-ID}= $name;
+    $dat{-NAME} = $dat{-ID}= $name;    
     $self->{'_cuts_after'} = $data[1];
 
     return %dat;
@@ -508,13 +895,15 @@ sub site {
     my $seq = $self->seq;
     my $cuts_after = $self->cuts_after;
     if($cuts_after > 0) {
-        return $seq->subseq(1, $self->cuts_after).'^'.$seq->subseq($self->cuts_after+1, $seq->length); 
+	if( $cuts_after >= $seq->length) {
+	    return $seq->seq.'^';
+	} else { 
+	    return $seq->subseq(1, $self->cuts_after).'^'.$seq->subseq($self->cuts_after+1, $seq->length); 
+	}
     } else {
         return $seq->seq;
     }
 }
-    
-
 
 
 =head1 seq
@@ -567,9 +956,9 @@ sub string {  my $self = shift; $self->{'_seq'}->seq; }
  Returns   : String
  Argument  : n/a
  Throws    : n/a
- Comments  : Delegates to the Bio::PrimarySeq.pm-derived object, but needs to get
-             out the string from it, as now Bio::PrimarySeq->revcom makes a Bio::PrimarySeq
-             object
+ Comments  : Delegates to the Bio::PrimarySeq.pm-derived object, but needs to
+             get out the string from it, as now Bio::PrimarySeq->revcom makes a
+             Bio::PrimarySeq object
 
 See Also   : L<seq()|seq>, L<string()|string>
 
@@ -593,8 +982,15 @@ sub revcom {  my $self = shift; $self->{'_seq'}->revcom->seq(); }
            : (Does not yet verify that it is derived from Bio::PrimarySeq.pm.)
  Comments  : Strategy relies on Perl's built-in split() function.
            : Since split removes the recognition pattern, the resulting
-           : fragments must be repaired after split()-ing.
+           : fragments are repaired after split()-ing.
+           : A side-effect of this is that for sites with ambiguous
+           : recognition sequence (i.e., containing N), the fragments
+           : will contain ambiguity characters instead of AGCT.
+           :
            : There is currently no support for partial digestions.
+           : There is currently no support for circular sequences.
+           : (This should just involve merging the first and last frag
+           : if $seqObj->is_circular returns true).
 
 =cut
 
@@ -608,7 +1004,7 @@ sub cut_seq {
 	$self->throw( "Can't cut sequence. Missing or invalid object".
 		      "seqObj: $seqObj");
     }
-    
+
     my $cuts_after = $self->{'_cuts_after'};
     my ($site_3prime_seq, $site_5prime_seq);
     my $reSeq = $self->seq;
@@ -634,7 +1030,7 @@ sub cut_seq {
     }
     $self->debug(sprintf("$ID: site seq: %s\n\n", $seq));
     $self->debug(sprintf("$ID: splitting %s\n\n",$reSeq->seq));
-    @re_frags = split(/$seq/, uc $seqObj->seq);
+    @re_frags = split(/$seq/i, $seqObj->seq);
 
     $self->debug("$ID: cut_seq, ".scalar @re_frags. " fragments.\n");
 
@@ -672,7 +1068,7 @@ sub cut_locations {
     my $seq = $seqobj->seq;
     study($seq);
     my @locations;
-    while( $seq =~ /($site)/g ) {
+    while( $seq =~ /($site)/ig ) {
         # $` is preceding string before pattern so length returns position
 	push @locations, length($`); 	
     }
@@ -849,7 +1245,3 @@ sub available_list {
 }
 
 1;
-
-	
-
-
