@@ -1,4 +1,4 @@
-# $Id: Node.pm,v 1.10.2.1 2002/07/07 20:30:54 jason Exp $
+# $Id: Node.pm,v 1.17.2.1 2003/02/04 18:21:08 jason Exp $
 #
 # BioPerl module for Bio::Tree::Node
 #
@@ -46,20 +46,17 @@ the Bioperl mailing list.  Your participation is much appreciated.
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
-email or the web:
+the web:
 
-  bioperl-bugs@bioperl.org
-  http://bioperl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich
 
 Email jason@bioperl.org
 
-Describe contact details here
-
 =head1 CONTRIBUTORS
 
-Additional contributors names and emails here
+Aaron Mackey amackey@virginia.edu
 
 =head1 APPENDIX
 
@@ -72,7 +69,7 @@ Internal methods are usually preceded with a _
 # Let the code begin...
 
 package Bio::Tree::Node;
-use vars qw(@ISA);
+use vars qw(@ISA $CREATIONORDER);
 use strict;
 
 use Bio::Root::Root;
@@ -80,18 +77,22 @@ use Bio::Tree::NodeI;
 
 @ISA = qw(Bio::Root::Root Bio::Tree::NodeI);
 
+BEGIN { 
+    $CREATIONORDER = 0;
+}
+
 =head2 new
 
  Title   : new
  Usage   : my $obj = new Bio::Tree::Node();
  Function: Builds a new Bio::Tree::Node object
  Returns : Bio::Tree::Node
- Args    : -left   => pointer to Left descendent (optional)
-           -right  => pointer to Right descenent (optional)
+ Args    : -left          => pointer to Left descendent (optional)
+           -right         => pointer to Right descenent (optional)
 	   -branch_length => branch length [integer] (optional)
-           -bootstrap => value   bootstrap value (string)
-           -desc      => description of node
-           -id        => unique id for node
+           -bootstrap     => value   bootstrap value (string)
+           -description   => description of node
+           -id            => unique id for node
 =cut
 
 sub new {
@@ -99,14 +100,21 @@ sub new {
 
   my $self = $class->SUPER::new(@args);
   my ($children, $branchlen,$id,
-      $bootstrap, $desc) = $self->_rearrange([qw(DESCENDENTS
+      $bootstrap, $desc,$d) = $self->_rearrange([qw(DESCENDENTS
 						 BRANCH_LENGTH
 						 ID
 						 BOOTSTRAP
 						 DESC
+						 DESCRIPTION
 						 )],
 					     @args);
-  $self->{'_desc'} = {};
+#  $self->{'_desc'} = {};
+  if( $d && $desc ) { 
+      $self->warn("can only accept -desc or -description, not both, accepting -description");
+      $desc = $d;
+  } elsif( defined $d && ! defined $desc ) {
+      $desc = $d;
+  }
   defined $desc && $self->description($desc);
   defined $bootstrap && $self->bootstrap($bootstrap);
   defined $id && $self->id($id);
@@ -120,6 +128,7 @@ sub new {
 	  $self->add_Descendent($c);
       }
   }
+  $self->_creation_id($CREATIONORDER++);
   return $self;
 }
 
@@ -130,40 +139,122 @@ sub new {
  Function: Adds a descendent to a node
  Returns : number of current descendents for this node
  Args    : Bio::Node::NodeI
+           boolean flag, true if you want to ignore the fact that you are
+           adding a second node with the same unique id (typically memory 
+           location reference in this implementation).  default is false and 
+           will throw an error if you try and overwrite an existing node.
 
 =cut
 
 sub add_Descendent{
-   my ($self,$node) = @_;
+   my ($self,$node,$ignoreoverwrite) = @_;
    return -1 if( ! defined $node ) ;
    if( ! $node->isa('Bio::Tree::NodeI') ) {
        $self->warn("Trying to add a Descendent who is not a Bio::Tree::NodeI");
        return -1;
    }
    # do we care about order?
-   $node->ancestor($self);
-   $self->{'_desc'}->{$node} = $node;
-   $self->invalidate_height;
+   $node->{'_ancestor'} = $self;
+   if( $self->{'_desc'}->{$node->internal_id} && ! $ignoreoverwrite ) {
+       $self->throw("Going to overwrite a node which is $node that is already stored here, set the ignore overwrite flag (parameter 2) to true to ignore this in the future");
+   }
+   
+   $self->{'_desc'}->{$node->internal_id} = $node; # is this safely unique - we've tested before at any rate??
+   
+   $self->invalidate_height();
+   
    return scalar keys %{$self->{'_desc'}};
 }
 
 
 =head2 each_Descendent
 
- Title   : each_Descendent
+ Title   : each_Descendent($sortby)
  Usage   : my @nodes = $node->each_Descendent;
  Function: all the descendents for this Node (but not their descendents
 					      i.e. not a recursive fetchall)
  Returns : Array of Bio::Tree::NodeI objects
- Args    : none
+ Args    : $sortby [optional] "height", "creation" or coderef to be used
+           to sort the order of children nodes.
 
 =cut
 
 sub each_Descendent{
-   my ($self) = @_;
+   my ($self, $sortby) = @_;
+
    # order can be based on branch length (and sub branchlength)
 
-   return sort { $a->height <=> $b->height } values %{$self->{'_desc'}};
+   $sortby ||= 'height';
+
+   if (ref $sortby eq 'CODE') {
+       return sort $sortby values %{$self->{'_desc'}};
+   } else  {
+       if ($sortby eq 'height') {
+	   return map { $_->[0] }
+		  sort { $a->[1] <=> $b->[1] || 
+			 $a->[2] <=> $b->[2] } 
+	       map { [$_, $_->height, $_->internal_id ] } 
+	   values %{$self->{'_desc'}};
+       } else {
+	   return map { $_->[0] }
+	          sort { $a->[1] <=> $b->[1] } 
+	          map { [$_, $_->height ] }
+	          values %{$self->{'_desc'}};	   
+       }
+   }
+}
+
+=head2 remove_Descendent
+
+ Title   : remove_Descendent
+ Usage   : $node->remove_Descedent($node_foo);
+ Function: Removes a specific node from being a Descendent of this node
+ Returns : nothing
+ Args    : An array of Bio::Node::NodeI objects which have be previously
+           passed to the add_Descendent call of this object.
+
+=cut
+
+sub remove_Descendent{
+   my ($self,@nodes) = @_;
+   foreach my $n ( @nodes ) { 
+       if( $self->{'_desc'}->{$n->internal_id} ) {
+	   $n->{'_ancestor'} = undef;
+	   $self->{'_desc'}->{$n->internal_id}->{'_ancestor'} = undef;
+	   delete $self->{'_desc'}->{$n->internal_id};
+	   
+       } else { 
+	   $self->debug(sprintf("no node %s (%s) listed as a descendent in this node %s (%s)\n",$n->id, $n,$self->id,$self));
+	   $self->debug("Descendents are " . join(',', keys %{$self->{'_desc'}})."\n");
+       }
+   }
+   1;
+}
+
+
+=head2 remove_all_Descendents
+
+ Title   : remove_all_Descendents
+ Usage   : $node->remove_All_Descendents()
+ Function: Cleanup the node's reference to descendents and reset
+           their ancestor pointers to undef, if you don't have a reference
+           to these objects after this call they will be cleanedup - so
+           a get_nodes from the Tree object would be a safe thing to do first
+ Returns : nothing
+ Args    : none
+
+
+=cut
+
+sub remove_all_Descendents{
+   my ($self) = @_;
+   # this won't cleanup the nodes themselves if you also have
+   # a copy/pointer of them (I think)...
+   while( my ($node,$val) = each %{ $self->{'_desc'} } ) {
+       $val->{'_ancestor'} = undef;
+   }
+   $self->{'_desc'} = {};
+   1;
 }
 
 =head2 get_Descendents
@@ -177,6 +268,9 @@ sub each_Descendent{
 
 =cut
 
+# implemented in the interface 
+
+
 =head2 ancestor
 
  Title   : ancestor
@@ -188,13 +282,8 @@ sub each_Descendent{
 =cut
 
 sub ancestor{
-   my ($self,$value) = @_;
-   if( defined $value) {
-       if(! $value->isa('Bio::Tree::NodeI') ) {
-	   $self->throw("Must specify a valid Bio::Tree::NodeI when setting the ancestor");
-       }
-       $self->{'_ancestor'} = $value;
-   }
+   my $self = shift;
+   $self->{'_ancestor'} = shift @_ if @_;
    return $self->{'_ancestor'};
 }
 
@@ -211,11 +300,9 @@ sub ancestor{
 =cut
 
 sub branch_length{
-    my ($self,$value) = @_;
-    if( defined $value) {
-	$self->{'branch_length'} = $value;
-    }
-    return $self->{'branch_length'};
+    my $self = shift;
+    $self->{'_branch_length'} = shift @_ if @_;
+    return $self->{'_branch_length'};
 }
 
 =head2 bootstrap
@@ -231,10 +318,8 @@ sub branch_length{
 =cut
 
 sub bootstrap{
-    my ($self,$value) = @_;
-    if( defined $value ) {
-       $self->{'_bootstrap'} = $value;
-    }
+    my $self = shift;
+    $self->{'_bootstrap'} = shift @_ if @_;
     return $self->{'_bootstrap'};
 }
 
@@ -251,11 +336,9 @@ sub bootstrap{
 =cut
 
 sub description{
-   my ($self,$value) = @_;
-   if( defined $value  ) {
-       $self->{'_description'} = $value;
-   }
-   return $self->{'_description'};
+    my $self = shift;
+    $self->{'_description'} = shift @_ if @_;
+    return $self->{'_description'};
 }
 
 =head2 id
@@ -271,11 +354,9 @@ sub description{
 =cut
 
 sub id{
-   my ($self,$value) = @_;
-   if( defined $value ) {
-       $self->{'_id'} = $value;
-   }
-   return $self->{'_id'};
+    my $self = shift;
+    $self->{'_id'} = shift @_ if @_;
+    return $self->{'_id'};
 }
 
 sub DESTROY {
@@ -284,13 +365,53 @@ sub DESTROY {
     $self->SUPER::DESTROY();
     if( defined $self->{'_desc'} &&
 	ref($self->{'_desc'}) =~ /ARRAY/i ) {
-	foreach my $n ( @{$self->{'_desc'}} ) {
-	    $n->DESTROY();
+	while( my ($nodeid,$node) = each %{ $self->{'_desc'} } ) {
+	    $node->{'_ancestor'} = undef; # insure no circular references
+	    $node->DESTROY();
+	    $node = undef;
 	}
 	$self->{'_desc'} = {};
     }
 }
 
+=head2 internal_id
+
+ Title   : internal_id
+ Usage   : my $internalid = $node->internal_id
+ Function: Returns the internal unique id for this Node
+           (a monotonically increasing number for this in-memory implementation
+            but could be a database determined unique id in other 
+	    implementations)
+ Returns : unique id
+ Args    : none
+
+=cut
+
+sub internal_id{
+   return $_[0]->_creation_id;
+}
+
+
+=head2 _creation_id
+
+ Title   : _creation_id
+ Usage   : $obj->_creation_id($newval)
+ Function: a private method signifying the internal creation order
+ Returns : value of _creation_id
+ Args    : newvalue (optional)
+
+
+=cut
+
+sub _creation_id{
+    my ($self,$value) = @_;
+    if( defined $value) {
+	$self->{'_creation_id'} = $value;
+    }
+    return $self->{'_creation_id'} || 0;
+}
+
+
 # The following methods are implemented by NodeI decorated interface
 
 =head2 is_Leaf
@@ -330,8 +451,6 @@ sub is_Leaf {
 
 =cut
 
-# this implementation differs from the interface because it caches
-
 sub height { 
     my ($self) = @_;
 
@@ -339,7 +458,7 @@ sub height {
     
     if( $self->is_Leaf ) { 
        if( !defined $self->branch_length ) { 
-	   $self->debug(sprintf("Trying to calculate height of a node when a Node (%s) has an undefined branch_length",$self->id || '?' ));
+	   $self->debug(sprintf("Trying to calculate height of a node when a Node (%s) has an undefined branch_length\n",$self->id || '?' ));
 	   return 0;
        }
        return $self->branch_length;
@@ -352,6 +471,7 @@ sub height {
    return ($self->{'_height'} = $max + ($self->branch_length || 1));
 }
 
+
 =head2 invalidate_height
 
  Title   : invalidate_height
