@@ -1,4 +1,4 @@
-# $Id: newick.pm,v 1.7.2.1 2002/04/21 14:30:22 jason Exp $
+# $Id: newick.pm,v 1.13.2.1 2003/02/04 18:16:18 jason Exp $
 #
 # BioPerl module for Bio::TreeIO::newick
 #
@@ -44,7 +44,7 @@ of the bugs and their resolution. Bug reports can be submitted via
 email or the web:
 
   bioperl-bugs@bioperl.org
-  http://bioperl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich
 
@@ -86,13 +86,13 @@ use Bio::Event::EventGeneratorI;
  Usage   : my $tree = $treeio->next_tree
  Function: Gets the next tree in the stream
  Returns : Bio::Tree::TreeI
- Args    :
+ Args    : none
 
 
 =cut
 
 sub next_tree{
-   my ($self,@args) = @_;
+   my ($self) = @_;
    local $/ = ";\n";
    return unless $_ = $self->_readline;
    s/\s+//g;
@@ -117,40 +117,47 @@ sub next_tree{
 	   $self->_eventHandler->start_element( {'Name' => 'tree'} );
 	   $lastevent = $ch;
        } elsif($ch eq ')' ) {
-	   if( $chars ) {
-	       if( $lastevent eq ':' ) {
-		   $self->_eventHandler->start_element( { 'Name' => 'branch_length'});
-		   $self->_eventHandler->characters($chars);
-		   $self->_eventHandler->end_element( {'Name' => 'branch_length'});
-	       } else { 
-		   $self->debug($chars."\n");
-		   $self->_eventHandler->start_element( { 'Name' => 'node' } );
-		   $self->_eventHandler->start_element( { 'Name' => 'id' } );
-		   $self->_eventHandler->characters($chars);
-		   $self->_eventHandler->end_element( { 'Name' => 'id' } );
-	       }	   	   
-	       $self->_eventHandler->end_element( {'Name' => 'node'} );
-	   }
-	   $self->_eventHandler->end_element( {'Name' => 'tree'} );
-	   $chars = '';
-	   $lastevent = $ch;
-       } elsif ( $ch eq ',' ) {
-	   if( $chars ) {
+	   if( length $chars ) {
 	       if( $lastevent eq ':' ) {
 		   $self->_eventHandler->start_element( { 'Name' => 'branch_length'});
 		   $self->_eventHandler->characters($chars);
 		   $self->_eventHandler->end_element( {'Name' => 'branch_length'});
 	       } else { 
+		   $self->debug("id with no branchlength is $chars\n");
 		   $self->_eventHandler->start_element( { 'Name' => 'node' } );
 		   $self->_eventHandler->start_element( { 'Name' => 'id' } );
 		   $self->_eventHandler->characters($chars);
 		   $self->_eventHandler->end_element( { 'Name' => 'id' } );
 	       }
-	       $self->_eventHandler->end_element( {'Name' => 'node'} );
+	       
+	   } else {
+	       $self->_eventHandler->start_element( {'Name' => 'node'} )
 	   }
+	   $self->_eventHandler->end_element( {'Name' => 'node'} );
+	   $self->_eventHandler->end_element( {'Name' => 'tree'} );
+	   $chars = '';
+	   $lastevent = $ch;
+       } elsif ( $ch eq ',' ) {
+	   if( length $chars ) {
+	       if( $lastevent eq ':' ) {
+		   $self->_eventHandler->start_element( { 'Name' => 'branch_length'});
+		   $self->_eventHandler->characters($chars);
+		   $self->_eventHandler->end_element( {'Name' => 'branch_length'});
+	       } else { 
+		   $self->debug("id with no branchlength is $chars\n");
+		   $self->_eventHandler->start_element( { 'Name' => 'node' } );
+		   $self->_eventHandler->start_element( { 'Name' => 'id' } );
+		   $self->_eventHandler->characters($chars);
+		   $self->_eventHandler->end_element( { 'Name' => 'id' } );
+	       }   
+	   } else {
+	       $self->_eventHandler->start_element( { 'Name' => 'node' } );
+	   }
+	   $self->_eventHandler->end_element( {'Name' => 'node'} );
 	   $chars = '';
 	   $lastevent = $ch;
        } elsif( $ch eq ':' ) {
+	   $self->debug("id with a branchlength coming is $chars\n");
 	   $self->_eventHandler->start_element( { 'Name' => 'node' } );
 	   $self->_eventHandler->start_element( { 'Name' => 'id' } );	   
 	   $self->_eventHandler->characters($chars);
@@ -159,7 +166,7 @@ sub next_tree{
 	   $lastevent = $ch;
        } else { 	   
 	   $chars .= $ch;
-       } 
+       }
    }
    return undef;
 }
@@ -175,13 +182,16 @@ sub next_tree{
 =cut
 
 sub write_tree{
-   my ($self,$tree) = @_;      
-   my @data = _write_tree_Helper($tree->get_root_node);
-   if($data[-1] !~ /\)$/ ) {
-       $data[0] = "(".$data[0];
-       $data[-1] .= ")";
+   my ($self,@trees) = @_;      
+   foreach my $tree( @trees ) {
+       my @data = _write_tree_Helper($tree->get_root_node);
+       if($data[-1] !~ /\)$/ ) {
+	   $data[0] = "(".$data[0];
+	   $data[-1] .= ")";
+       }
+       $self->_print(join(',', @data), ";\n");   
    }
-   $self->_print(join(',', @data), ";\n");   
+   $self->flush if $self->_flush_on_write && defined $self->_fh;
    return;
 }
 
@@ -195,11 +205,10 @@ sub _write_tree_Helper {
 	push @data, _write_tree_Helper($n);
     }
 
-    if( @data > 1 && 
-	$node->branch_length) {
+    if( @data > 1 ) {
 	$data[0] = "(" . $data[0];
 	$data[-1] .= ")";
-	$data[-1] .= ":". $node->branch_length;
+	$data[-1] .= ":". $node->branch_length if( $node->branch_length);
     } else {
 	if( defined $node->id || defined $node->branch_length ) { 
 	    push @data, sprintf("%s%s",
